@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -161,6 +161,17 @@ ShaderConstant BufferFormatter::ParseFormatString(const QString &formatString, u
 
       QString varName = structMatch.captured(3);
 
+      QString arrayDim = structMatch.captured(4).trimmed();
+      uint32_t arrayCount = 1;
+      if(!arrayDim.isEmpty())
+      {
+        arrayDim = arrayDim.mid(1, arrayDim.count() - 2);
+        bool ok = false;
+        arrayCount = arrayDim.toUInt(&ok);
+        if(!ok)
+          arrayCount = 1;
+      }
+
       if(isPointer)
       {
         // if not tight packing, align up to pointer size
@@ -173,23 +184,13 @@ ShaderConstant BufferFormatter::ParseFormatString(const QString &formatString, u
         el.type.descriptor.type = VarType::ULong;
         el.type.descriptor.displayAsHex = true;
         el.type.descriptor.arrayByteStride = 8;
+        el.type.descriptor.elements = arrayCount;
 
         cur->offset += 8;
         cur->structDef.type.members.push_back(el);
       }
       else
       {
-        QString arrayDim = structMatch.captured(4).trimmed();
-        uint32_t arrayCount = 1;
-        if(!arrayDim.isEmpty())
-        {
-          arrayDim = arrayDim.mid(1, arrayDim.count() - 2);
-          bool ok = false;
-          arrayCount = arrayDim.toUInt(&ok);
-          if(!ok)
-            arrayCount = 1;
-        }
-
         // cbuffer packing rules, structs are always float4 base aligned
         if(!tightPacking)
           cur->offset = (cur->offset + 0xFU) & (~0xFU);
@@ -599,66 +600,58 @@ QString BufferFormatter::GetBufferFormatString(const ShaderResource &res,
 
   if(!res.variableType.members.empty())
   {
+    QList<QString> declaredStructs;
     if(m_API == GraphicsAPI::Vulkan || m_API == GraphicsAPI::OpenGL)
     {
       const rdcarray<ShaderConstant> &members = res.variableType.members;
 
-      format += QFormatStr("struct %1\n{\n").arg(res.name);
-
-      // GL/Vulkan allow fixed-sized members before the array-of-structs. This can't be
-      // represented in a buffer format so we skip it
-      if(members.count() > 1)
+      // if there is only one member in the root array, we can just call DeclareStruct directly
+      if(members.count() <= 1)
       {
-        format += tr("    // members skipped as they are fixed size:\n");
-        baseByteOffset += members.back().byteOffset;
+        format = DeclareStruct(declaredStructs, res.name, members, 0, QString());
       }
-
-      QString varTypeName;
-      QString comment = lit("// ");
-      for(int i = 0; i < members.count(); i++)
+      else
       {
-        QString arraySize;
-        if(members[i].type.descriptor.elements > 1)
-          arraySize = QFormatStr("[%1]").arg(members[i].type.descriptor.elements);
+        // otherwise we need to build up the comment indicating which fixed-size members we skipped
+        QString fixedPrefixString = tr("    // members skipped as they are fixed size:\n");
+        baseByteOffset += members.back().byteOffset;
 
-        varTypeName = members[i].type.descriptor.name;
-
-        if(i + 1 == members.count())
+        // list each member before the last, commented out.
+        for(int i = 0; i < members.count() - 1; i++)
         {
-          comment.clear();
-          arraySize.clear();
+          QString arraySize;
+          if(members[i].type.descriptor.elements > 1 && members[i].type.descriptor.elements != ~0U)
+            arraySize = QFormatStr("[%1]").arg(members[i].type.descriptor.elements);
 
-          if(members.count() > 1)
-            format +=
-                lit("    // final array struct @ byte offset %1\n").arg(members.back().byteOffset);
+          QString varName = members[i].name;
 
-          // give GL nameless structs a better name
-          if(varTypeName.isEmpty() || varTypeName == lit("struct"))
-            varTypeName = lit("root_struct");
+          if(varName.isEmpty())
+            varName = QFormatStr("_child%1").arg(i);
+
+          fixedPrefixString += QFormatStr("    // %1 %2%3;\n")
+                                   .arg(members[i].type.descriptor.name)
+                                   .arg(varName)
+                                   .arg(arraySize);
         }
 
-        QString varName = members[i].name;
+        fixedPrefixString +=
+            lit("    // final array struct @ byte offset %1\n").arg(members.back().byteOffset);
 
-        if(varName.isEmpty())
-          varName = QFormatStr("_child%1").arg(i);
+        // construct a fake list of members with only the last arrayed one, to pass to DeclareStruct
+        rdcarray<ShaderConstant> fakeLastMember;
+        fakeLastMember.push_back(members.back());
+        // rebase offset of this member to 0 so that DeclareStruct doesn't think any padding is
+        // needed
+        fakeLastMember[0].byteOffset = 0;
 
-        format +=
-            QFormatStr("    %1%2 %3%4;\n").arg(comment).arg(varTypeName).arg(varName).arg(arraySize);
-      }
-
-      format += lit("}");
-
-      // if the last member is a struct, declare it
-      if(!members.back().type.members.isEmpty())
-      {
-        format = DeclareStruct(varTypeName, members.back().type.members,
-                               members.back().type.descriptor.arrayByteStride) +
-                 lit("\n") + format;
+        format = DeclareStruct(declaredStructs, res.name, fakeLastMember,
+                               fakeLastMember[0].type.descriptor.arrayByteStride, fixedPrefixString);
       }
     }
     else
     {
-      format = DeclareStruct(res.variableType.descriptor.name, res.variableType.members, 0);
+      format = DeclareStruct(declaredStructs, res.variableType.descriptor.name,
+                             res.variableType.members, 0, QString());
     }
   }
   else
@@ -771,12 +764,34 @@ uint32_t BufferFormatter::GetVarSize(const ShaderConstant &var)
       size = var.type.descriptor.matrixByteStride * var.type.descriptor.columns;
   }
 
-  if(var.type.descriptor.elements > 1)
+  if(!var.type.members.empty())
+    size = GetStructVarSize(var.type.members);
+
+  if(var.type.descriptor.elements > 1 && var.type.descriptor.elements != ~0U)
     size *= var.type.descriptor.elements;
 
   return size;
 }
 
+uint32_t BufferFormatter::GetStructVarSize(const rdcarray<ShaderConstant> &members)
+{
+  uint32_t lastMemberStart = 0;
+
+  const ShaderConstant *lastChild = &members.back();
+
+  lastMemberStart += lastChild->byteOffset;
+  while(!lastChild->type.members.isEmpty())
+  {
+    if(lastChild->type.descriptor.elements != ~0U)
+      lastMemberStart += (qMax(lastChild->type.descriptor.elements, 1U) - 1) *
+                         lastChild->type.descriptor.arrayByteStride;
+    lastChild = &lastChild->type.members.back();
+    lastMemberStart += lastChild->byteOffset;
+  }
+
+  return lastMemberStart + GetVarSize(*lastChild);
+}
+
 QString BufferFormatter::DeclarePaddingBytes(uint32_t bytes)
 {
   if(bytes == 0)
@@ -805,12 +820,14 @@ QString BufferFormatter::DeclarePaddingBytes(uint32_t bytes)
 
 QString BufferFormatter::DeclareStruct(QList<QString> &declaredStructs, const QString &name,
                                        const rdcarray<ShaderConstant> &members,
-                                       uint32_t requiredByteStride)
+                                       uint32_t requiredByteStride, QString innerSkippedPrefixString)
 {
   QString ret;
 
   ret = lit("struct %1\n{\n").arg(name);
 
+  ret += innerSkippedPrefixString;
+
   uint32_t offset = 0;
 
   for(int i = 0; i < members.count(); i++)
@@ -824,14 +841,14 @@ QString BufferFormatter::DeclareStruct(QList<QString> &declaredStructs, const QS
     offset = members[i].byteOffset + GetVarSize(members[i]);
 
     QString arraySize;
-    if(members[i].type.descriptor.elements > 1)
+    if(members[i].type.descriptor.elements > 1 && members[i].type.descriptor.elements != ~0U)
       arraySize = QFormatStr("[%1]").arg(members[i].type.descriptor.elements);
 
     QString varTypeName = members[i].type.descriptor.name;
 
     if(members[i].type.descriptor.pointerTypeID != ~0U)
     {
-      const ShaderVariableType &pointeeType =
+      const ShaderConstantType &pointeeType =
           PointerTypeRegistry::GetTypeDescriptor(members[i].type.descriptor.pointerTypeID);
 
       varTypeName = pointeeType.descriptor.name;
@@ -840,7 +857,7 @@ QString BufferFormatter::DeclareStruct(QList<QString> &declaredStructs, const QS
       {
         declaredStructs.push_back(varTypeName);
         ret = DeclareStruct(declaredStructs, varTypeName, pointeeType.members,
-                            pointeeType.descriptor.arrayByteStride) +
+                            pointeeType.descriptor.arrayByteStride, QString()) +
               lit("\n") + ret;
       }
 
@@ -858,7 +875,7 @@ QString BufferFormatter::DeclareStruct(QList<QString> &declaredStructs, const QS
       {
         declaredStructs.push_back(varTypeName);
         ret = DeclareStruct(declaredStructs, varTypeName, members[i].type.members,
-                            members[i].type.descriptor.arrayByteStride) +
+                            members[i].type.descriptor.arrayByteStride, QString()) +
               lit("\n") + ret;
       }
     }
@@ -921,20 +938,7 @@ QString BufferFormatter::DeclareStruct(QList<QString> &declaredStructs, const QS
 
   if(requiredByteStride > 0)
   {
-    uint32_t lastMemberStart = 0;
-
-    const ShaderConstant *lastChild = &members.back();
-
-    lastMemberStart += lastChild->byteOffset;
-    while(!lastChild->type.members.isEmpty())
-    {
-      lastMemberStart += (qMax(lastChild->type.descriptor.elements, 1U) - 1) *
-                         lastChild->type.descriptor.arrayByteStride;
-      lastChild = &lastChild->type.members.back();
-      lastMemberStart += lastChild->byteOffset;
-    }
-
-    const uint32_t structEnd = lastMemberStart + GetVarSize(*lastChild);
+    const uint32_t structEnd = GetStructVarSize(members);
 
     if(requiredByteStride > structEnd)
       ret += lit("    ") + DeclarePaddingBytes(requiredByteStride - structEnd);
@@ -951,7 +955,7 @@ QString BufferFormatter::DeclareStruct(const QString &name, const rdcarray<Shade
                                        uint32_t requiredByteStride)
 {
   QList<QString> declaredStructs;
-  return DeclareStruct(declaredStructs, name, members, requiredByteStride);
+  return DeclareStruct(declaredStructs, name, members, requiredByteStride, QString());
 }
 
 void SetInterpretedResourceFormat(ShaderConstant &elem, ResourceFormatType interpretType,
@@ -1023,7 +1027,7 @@ static void FillShaderVarData(ShaderVariable &var, const ShaderConstant &elem, c
   if(objs.isEmpty())
   {
     var.name = "-";
-    memset(var.value.dv, 0, sizeof(var.value.dv));
+    var.value = ShaderValue();
     return;
   }
 
@@ -1041,21 +1045,21 @@ static void FillShaderVarData(ShaderVariable &var, const ShaderConstant &elem, c
       src++;
 
       if(var.type == VarType::Double)
-        var.value.dv[dst] = o.toDouble();
+        var.value.f64v[dst] = o.toDouble();
       if(var.type == VarType::Float || var.type == VarType::Half)
-        var.value.fv[dst] = o.toFloat();
+        var.value.f32v[dst] = o.toFloat();
       else if(var.type == VarType::ULong)
         var.value.u64v[dst] = o.toULongLong();
       else if(var.type == VarType::SLong)
         var.value.s64v[dst] = o.toLongLong();
       else if(var.type == VarType::Bool)
-        var.value.uv[dst] = o.toBool() ? 1 : 0;
+        var.value.u32v[dst] = o.toBool() ? 1 : 0;
       else if(var.type == VarType::UInt || var.type == VarType::UShort || var.type == VarType::UByte)
-        var.value.uv[dst] = o.toUInt();
+        var.value.u32v[dst] = o.toUInt();
       else if(var.type == VarType::SInt || var.type == VarType::SShort || var.type == VarType::SByte)
-        var.value.iv[dst] = o.toInt();
+        var.value.s32v[dst] = o.toInt();
       else
-        var.value.fv[dst] = o.toFloat();
+        var.value.f32v[dst] = o.toFloat();
     }
   }
 }
@@ -1077,7 +1081,7 @@ ShaderVariable InterpretShaderVar(const ShaderConstant &elem, const byte *data,
   {
     ret.rows = ret.columns = 0;
 
-    if(elem.type.descriptor.elements > 1)
+    if(elem.type.descriptor.elements > 1 && elem.type.descriptor.elements != ~0U)
     {
       rdcarray<ShaderVariable> arrayElements;
 
@@ -1116,7 +1120,7 @@ ShaderVariable InterpretShaderVar(const ShaderConstant &elem, const byte *data,
       ret.members = members;
     }
   }
-  else if(elem.type.descriptor.elements > 1)
+  else if(elem.type.descriptor.elements > 1 && elem.type.descriptor.elements != ~0U)
   {
     rdcarray<ShaderVariable> arrayElements;
 
@@ -1239,7 +1243,7 @@ inline T readObj(const byte *&data, const byte *end, bool &ok)
   return ret;
 }
 
-QVariantList GetVariants(ResourceFormat format, const ShaderVariableDescriptor &varDesc,
+QVariantList GetVariants(ResourceFormat format, const ShaderConstantDescriptor &varDesc,
                          const byte *&data, const byte *end)
 {
   QVariantList ret;
@@ -1635,9 +1639,9 @@ QString RowString(const ShaderVariable &v, uint32_t row, VarType type)
     return ToQStr(v.GetPointer());
 
   if(type == VarType::Double)
-    return RowValuesToString((int)v.columns, v.displayAsHex, v.value.dv[row * v.columns + 0],
-                             v.value.dv[row * v.columns + 1], v.value.dv[row * v.columns + 2],
-                             v.value.dv[row * v.columns + 3]);
+    return RowValuesToString((int)v.columns, v.displayAsHex, v.value.f64v[row * v.columns + 0],
+                             v.value.f64v[row * v.columns + 1], v.value.f64v[row * v.columns + 2],
+                             v.value.f64v[row * v.columns + 3]);
   else if(type == VarType::SLong)
     return RowValuesToString((int)v.columns, v.displayAsHex, v.value.s64v[row * v.columns + 0],
                              v.value.s64v[row * v.columns + 1], v.value.s64v[row * v.columns + 2],
@@ -1647,23 +1651,23 @@ QString RowString(const ShaderVariable &v, uint32_t row, VarType type)
                              v.value.u64v[row * v.columns + 1], v.value.u64v[row * v.columns + 2],
                              v.value.u64v[row * v.columns + 3]);
   else if(type == VarType::SInt || type == VarType::SShort || type == VarType::SByte)
-    return RowValuesToString((int)v.columns, v.displayAsHex, v.value.iv[row * v.columns + 0],
-                             v.value.iv[row * v.columns + 1], v.value.iv[row * v.columns + 2],
-                             v.value.iv[row * v.columns + 3]);
+    return RowValuesToString((int)v.columns, v.displayAsHex, v.value.s32v[row * v.columns + 0],
+                             v.value.s32v[row * v.columns + 1], v.value.s32v[row * v.columns + 2],
+                             v.value.s32v[row * v.columns + 3]);
   else if(type == VarType::UInt || type == VarType::UShort || type == VarType::UByte)
-    return RowValuesToString((int)v.columns, v.displayAsHex, v.value.uv[row * v.columns + 0],
-                             v.value.uv[row * v.columns + 1], v.value.uv[row * v.columns + 2],
-                             v.value.uv[row * v.columns + 3]);
+    return RowValuesToString((int)v.columns, v.displayAsHex, v.value.u32v[row * v.columns + 0],
+                             v.value.u32v[row * v.columns + 1], v.value.u32v[row * v.columns + 2],
+                             v.value.u32v[row * v.columns + 3]);
   else if(type == VarType::Bool)
     return RowValuesToString((int)v.columns, v.displayAsHex,
-                             v.value.uv[row * v.columns + 0] ? true : false,
-                             v.value.uv[row * v.columns + 1] ? true : false,
-                             v.value.uv[row * v.columns + 2] ? true : false,
-                             v.value.uv[row * v.columns + 3] ? true : false);
+                             v.value.u32v[row * v.columns + 0] ? true : false,
+                             v.value.u32v[row * v.columns + 1] ? true : false,
+                             v.value.u32v[row * v.columns + 2] ? true : false,
+                             v.value.u32v[row * v.columns + 3] ? true : false);
   else
-    return RowValuesToString((int)v.columns, v.displayAsHex, v.value.fv[row * v.columns + 0],
-                             v.value.fv[row * v.columns + 1], v.value.fv[row * v.columns + 2],
-                             v.value.fv[row * v.columns + 3]);
+    return RowValuesToString((int)v.columns, v.displayAsHex, v.value.f32v[row * v.columns + 0],
+                             v.value.f32v[row * v.columns + 1], v.value.f32v[row * v.columns + 2],
+                             v.value.f32v[row * v.columns + 3]);
 }
 
 QString VarString(const ShaderVariable &v)
