@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -28,6 +28,7 @@
 #include <QDirIterator>
 #include <QElapsedTimer>
 #include <QFileInfo>
+#include <QFileSystemWatcher>
 #include <QLabel>
 #include <QMenu>
 #include <QMessageBox>
@@ -43,6 +44,7 @@
 #include "Windows/ConstantBufferPreviewer.h"
 #include "Windows/DebugMessageView.h"
 #include "Windows/Dialogs/CaptureDialog.h"
+#include "Windows/Dialogs/CrashDialog.h"
 #include "Windows/Dialogs/LiveCapture.h"
 #include "Windows/Dialogs/SettingsDialog.h"
 #include "Windows/EventBrowser.h"
@@ -53,6 +55,7 @@
 #include "Windows/PixelHistoryView.h"
 #include "Windows/PythonShell.h"
 #include "Windows/ResourceInspector.h"
+#include "Windows/ShaderMessageViewer.h"
 #include "Windows/ShaderViewer.h"
 #include "Windows/StatisticsViewer.h"
 #include "Windows/TextureViewer.h"
@@ -81,7 +84,7 @@ CaptureContext::CaptureContext(PersistantConfig &cfg) : m_Config(cfg)
   m_CurVulkanPipelineState = NULL;
   m_CurPipelineState = &m_DummyPipelineState;
 
-  m_Drawcalls = &m_EmptyDraws;
+  m_Actions = &m_EmptyActions;
 
   m_StructuredFile = &m_DummySDFile;
 
@@ -96,8 +99,109 @@ CaptureContext::CaptureContext(PersistantConfig &cfg) : m_Config(cfg)
 
   m_MainWindow->LoadInitialLayout();
 
+  m_Replay.SetFatalErrorCallback([this]() {
+    GUIInvoke::call(m_MainWindow, [this]() {
+      RefreshUIStatus({}, true, true);
+
+      ReplayStatus err = GetFatalError();
+
+      QString title;
+      QString text;
+
+      if(err == ReplayStatus::RemoteServerConnectionLost)
+      {
+        QString serverName = Replay().CurrentRemote().Name();
+
+        title = tr("Connection lost to %1").arg(serverName);
+        text =
+            tr("Connection to %1 was lost while analysing capture.\n"
+               "This is most commonly caused by a crash, but that can't be verified and it may be "
+               "a connection issue.")
+                .arg(serverName);
+      }
+      else if(err == ReplayStatus::ReplayDeviceLost)
+      {
+        title = tr("Device Lost error");
+        text =
+            tr("The capture analysis encountered a device lost error. This may be due to an "
+               "application bug, a RenderDoc bug, or insufficient resources on the system to "
+               "analyse the capture.\n\n"
+               "It is recommended that you run your application with API validation enabled, as "
+               "API use errors can cause this kind of problem.");
+      }
+      else if(err == ReplayStatus::ReplayOutOfMemory)
+      {
+        title = tr("Out of GPU memory");
+        text = tr(
+            "While analysing the capture it ran out of GPU memory. This is most likely caused by "
+            "insufficient resources on the system to analyse the capture, which is possible if "
+            "there is memory pressure from other applications or if the capture is heavyweight.");
+      }
+      else
+      {
+        title = tr("Unrecoverable error");
+        text = tr("While analysing the capture we encountered an unrecoverable error:\n\n%1")
+                   .arg(QString(ToStr(err)));
+      }
+
+      text +=
+          tr("\n\n"
+             "This error is not recoverable and the analysis cannot continue. The UI will be "
+             "non-functional until the capture is closed and reloaded.\n\n");
+
+      bool add_report = false;
+
+      if(CrashDialog::HasCaptureReady(m_Config))
+      {
+        text += tr(
+            "If you think this may be a RenderDoc bug please click the button below to report it, "
+            "but note that this will require you to upload the capture for reproduction as "
+            "otherwise it is impossible to tell what the problem may be.");
+
+        add_report = true;
+      }
+      else if(CrashDialog::CaptureTooLarge(m_Config))
+      {
+        text +=
+            tr("<html>Your capture is too lage to upload as a crash report so this can't be "
+               "automatically reported. "
+               "Please email me at <a "
+               "href=\"mailto:baldurk@baldurk.org?subject=RenderDoc%20Unrecoverable%20error\">"
+               "baldurk@baldurk.org</a> with information and I can help investigate.</html>");
+      }
+      else
+      {
+        text += tr("The capture must be saved locally if you want to report this as a bug. ");
+
+        if(Replay().CurrentRemote().IsConnected())
+        {
+          text +=
+              tr("Before closing the capture you can save it to disk and manually report a bug. "
+                 "Please include the capture, or else it will be impossible to tell what the "
+                 "problem may be.");
+        }
+        else
+        {
+          text += tr("You will need to reconnect to the remote server to save the capture.");
+        }
+      }
+
+      QMessageBox mb(QMessageBox::Critical, title, text, QMessageBox::Ok, m_MainWindow);
+      mb.setDefaultButton(QMessageBox::NoButton);
+      QPushButton *report = NULL;
+      if(add_report)
+        report = mb.addButton(tr("Report bug"), QMessageBox::AcceptRole);
+      RDDialog::show(&mb);
+
+      if(mb.clickedButton() == (QAbstractButton *)report)
+      {
+        m_MainWindow->sendErrorReport(true);
+      }
+    });
+  });
+
   {
-    QDir dir(configFilePath("extensions"));
+    QDir dir(ConfigFilePath("extensions"));
 
     if(!dir.exists())
       dir.mkpath(dir.absolutePath());
@@ -124,7 +228,8 @@ CaptureContext::~CaptureContext()
   delete m_MainWindow;
 }
 
-void CaptureContext::Begin(QString paramFilename, QString remoteHost, uint32_t remoteIdent, bool temp)
+void CaptureContext::Begin(QString paramFilename, QString remoteHost, uint32_t remoteIdent,
+                           bool temp, QString scriptFilename)
 {
   m_MainWindow->show();
 
@@ -145,6 +250,14 @@ void CaptureContext::Begin(QString paramFilename, QString remoteHost, uint32_t r
         m_MainWindow->takeCaptureOwnership();
     }
   }
+
+  if(!scriptFilename.isEmpty())
+  {
+    ShowPythonShell();
+
+    if(GetPythonShell()->LoadScriptFromFilename(scriptFilename))
+      GetPythonShell()->RunScript();
+  }
 }
 
 bool CaptureContext::isRunning()
@@ -710,10 +823,11 @@ void CaptureContext::LoadCapture(const rdcstr &captureFile, const ReplayOptions
   m_LoadInProgress = true;
 
   if(local)
-  {
     m_Config.CrashReport_LastOpenedCapture = origFilename;
-    m_Config.Save();
-  }
+  else
+    m_Config.CrashReport_LastOpenedCapture = QString();
+
+  m_Config.Save();
 
   bool newCapture = (!temporary && !Config().RecentCaptureFiles.contains(origFilename));
 
@@ -728,10 +842,23 @@ void CaptureContext::LoadCapture(const rdcstr &captureFile, const ReplayOptions
   QElapsedTimer loadTimer;
   loadTimer.start();
 
-  ShowProgressDialog(m_MainWindow, tr("Loading Capture: %1").arg(origFilename),
+  ShowProgressDialog(m_MainWindow, tr("Loading Capture: %1").arg(QFileInfo(origFilename).fileName()),
                      [this]() { return !m_LoadInProgress; },
                      [this]() { return UpdateLoadProgress(); });
 
+  if(local)
+  {
+    m_Watcher = new QFileSystemWatcher({captureFile}, GetMainWindow()->Widget());
+
+    QObject::connect(m_Watcher, &QFileSystemWatcher::fileChanged, [this]() {
+      Replay().AsyncInvoke([this](IReplayController *r) {
+        r->FileChanged();
+        r->SetFrameEvent(m_EventID, true);
+        GUIInvoke::call(GetMainWindow()->Widget(), [this]() { RefreshUIStatus({}, true, true); });
+      });
+    });
+  }
+
 #if defined(RELEASE)
   ANALYTIC_ADDAVG(Performance.LoadTime, double(loadTimer.nsecsElapsed() * 1.0e-9));
 #endif
@@ -767,10 +894,10 @@ void CaptureContext::LoadCapture(const rdcstr &captureFile, const ReplayOptions
     rdcarray<ICaptureViewer *> viewers(m_CaptureViewers);
 
     // make sure we're on a consistent event before invoking viewer forms
-    if(m_LastDrawcall)
-      SetEventID(viewers, m_LastDrawcall->eventId, m_LastDrawcall->eventId, true);
-    else if(!m_Drawcalls->empty())
-      SetEventID(viewers, m_Drawcalls->back().eventId, m_Drawcalls->back().eventId, true);
+    if(m_LastAction)
+      SetEventID(viewers, m_LastAction->eventId, m_LastAction->eventId, true);
+    else if(!m_Actions->empty())
+      SetEventID(viewers, m_Actions->back().eventId, m_Actions->back().eventId, true);
 
     GUIInvoke::blockcall(m_MainWindow, [&viewers]() {
       // notify all the registers viewers that a capture has been loaded
@@ -842,9 +969,9 @@ void CaptureContext::LoadCaptureThreaded(const QString &captureFile, const Repla
 
   m_EventID = 0;
 
-  m_FirstDrawcall = m_LastDrawcall = NULL;
+  m_FirstAction = m_LastAction = NULL;
 
-  // fetch initial data like drawcalls, textures and buffers
+  // fetch initial data like actions, textures and buffers
   m_Replay.BlockInvoke([this](IReplayController *r) {
     if(Config().EventBrowser_AddFake)
       r->AddFakeMarkers();
@@ -858,15 +985,15 @@ void CaptureContext::LoadCaptureThreaded(const QString &captureFile, const Repla
 
     m_PostloadProgress = 0.2f;
 
-    m_Drawcalls = &r->GetDrawcalls();
+    m_Actions = &r->GetRootActions();
 
-    m_FirstDrawcall = &m_Drawcalls->at(0);
-    while(!m_FirstDrawcall->children.empty())
-      m_FirstDrawcall = &m_FirstDrawcall->children[0];
+    m_FirstAction = &m_Actions->at(0);
+    while(!m_FirstAction->children.empty())
+      m_FirstAction = &m_FirstAction->children[0];
 
-    m_LastDrawcall = &m_Drawcalls->back();
-    while(!m_LastDrawcall->children.empty())
-      m_LastDrawcall = &m_LastDrawcall->children.back();
+    m_LastAction = &m_Actions->back();
+    while(!m_LastAction->children.empty())
+      m_LastAction = &m_LastAction->children.back();
 
     m_PostloadProgress = 0.4f;
 
@@ -919,6 +1046,8 @@ void CaptureContext::LoadCaptureThreaded(const QString &captureFile, const Repla
 
 #elif defined(RENDERDOC_PLATFORM_APPLE)
     m_CurWinSystem = WindowingSystem::MacOS;
+#elif defined(RENDERDOC_PLATFORM_GGP)
+    m_CurWinSystem = WindowingSystem::GGP;
 #endif
 
     m_StructuredFile = &r->GetStructuredFile();
@@ -1095,7 +1224,7 @@ void CaptureContext::RecompressCapture()
   {
     // for remote files we open a new short-lived handle on the temporary file
     tempCap = cap = RENDERDOC_OpenCaptureFile();
-    cap->OpenFile(tempFilename.toUtf8().data(), "rdc", NULL);
+    cap->OpenFile(tempFilename, "rdc", NULL);
   }
 
   if(!cap)
@@ -1127,7 +1256,7 @@ void CaptureContext::RecompressCapture()
   float progress = 0.0f;
 
   LambdaThread *th = new LambdaThread([cap, destFilename, &progress]() {
-    cap->Convert(destFilename.toUtf8().data(), "rdc", NULL, [&progress](float p) { progress = p; });
+    cap->Convert(destFilename, "rdc", NULL, [&progress](float p) { progress = p; });
   });
   th->setName(lit("RecompressCapture"));
   th->start();
@@ -1155,7 +1284,7 @@ void CaptureContext::RecompressCapture()
     QFile::rename(destFilename, GetCaptureFilename());
 
     // and re-open
-    cap->OpenFile(GetCaptureFilename().c_str(), "rdc", NULL);
+    cap->OpenFile(GetCaptureFilename(), "rdc", NULL);
   }
   else
   {
@@ -1213,7 +1342,7 @@ bool CaptureContext::SaveCaptureTo(const rdcstr &captureFile)
         if(capFile)
         {
           // this will overwrite
-          success = capFile->CopyFileTo(captureFile.c_str());
+          success = capFile->CopyFileTo(captureFile);
         }
         else
         {
@@ -1272,6 +1401,9 @@ void CaptureContext::CloseCapture()
   if(!m_CaptureLoaded)
     return;
 
+  delete m_Watcher;
+  m_Watcher = NULL;
+
   delete m_RGP;
   m_RGP = NULL;
 
@@ -1281,22 +1413,23 @@ void CaptureContext::CloseCapture()
 
   m_CaptureFile = QString();
 
-  memset(&m_APIProps, 0, sizeof(m_APIProps));
-  memset(&m_FrameInfo, 0, sizeof(m_FrameInfo));
+  m_APIProps = APIProperties();
+  m_FrameInfo = FrameDescription();
   m_Buffers.clear();
   m_BufferList.clear();
   m_Textures.clear();
   m_TextureList.clear();
   m_Resources.clear();
   m_ResourceList.clear();
-  m_ReplacedResources.clear();
+  m_OrigToReplacedResources.clear();
+  m_ReplacedToOrigResources.clear();
 
   m_CustomNames.clear();
   m_Bookmarks.clear();
   m_Notes.clear();
 
-  m_Drawcalls = &m_EmptyDraws;
-  m_FirstDrawcall = m_LastDrawcall = NULL;
+  m_Actions = &m_EmptyActions;
+  m_FirstAction = m_LastAction = NULL;
 
   m_CurD3D11PipelineState = NULL;
   m_CurD3D12PipelineState = NULL;
@@ -1341,7 +1474,7 @@ bool CaptureContext::ImportCapture(const CaptureFileFormat &fmt, const rdcstr &i
 
     ICaptureFile *file = RENDERDOC_OpenCaptureFile();
 
-    status = file->OpenFile(importfile.c_str(), ext.toUtf8().data(),
+    status = file->OpenFile(importfile, ext.toUtf8().data(),
                             [&progress](float p) { progress = p * 0.5f; });
 
     if(status != ReplayStatus::Succeeded)
@@ -1351,8 +1484,8 @@ bool CaptureContext::ImportCapture(const CaptureFileFormat &fmt, const rdcstr &i
       return;
     }
 
-    status = file->Convert(rdcfile.c_str(), "rdc", NULL,
-                           [&progress](float p) { progress = 0.5f + p * 0.5f; });
+    status =
+        file->Convert(rdcfile, "rdc", NULL, [&progress](float p) { progress = 0.5f + p * 0.5f; });
     file->Shutdown();
   });
   th->setName(lit("ImportCapture"));
@@ -1404,7 +1537,7 @@ void CaptureContext::ExportCapture(const CaptureFileFormat &fmt, const rdcstr &e
   if(!file)
   {
     local = file = RENDERDOC_OpenCaptureFile();
-    status = file->OpenFile(m_CaptureFile.toUtf8().data(), "rdc", NULL);
+    status = file->OpenFile(m_CaptureFile, "rdc", NULL);
   }
 
   QString filename = QFileInfo(m_CaptureFile).fileName();
@@ -1428,8 +1561,7 @@ void CaptureContext::ExportCapture(const CaptureFileFormat &fmt, const rdcstr &e
   float progress = 0.0f;
 
   LambdaThread *th = new LambdaThread([file, sdfile, ext, exportfile, &progress, &status]() {
-    status = file->Convert(exportfile.c_str(), ext.toUtf8().data(), sdfile,
-                           [&progress](float p) { progress = p; });
+    status = file->Convert(exportfile, ext, sdfile, [&progress](float p) { progress = p; });
   });
   th->setName(lit("ExportCapture"));
   th->start();
@@ -1467,7 +1599,7 @@ void CaptureContext::SetEventID(const rdcarray<ICaptureViewer *> &exclude, uint3
   if(!IsCaptureLoaded())
     return;
 
-  if(eventId > m_LastDrawcall->eventId)
+  if(eventId > m_LastAction->eventId)
   {
     qCritical() << "Invalid EID being selected " << eventId;
     return;
@@ -1478,15 +1610,36 @@ void CaptureContext::SetEventID(const rdcarray<ICaptureViewer *> &exclude, uint3
   uint32_t prevEventID = m_EventID;
   m_EventID = eventId;
 
-  m_Replay.BlockInvoke([this, eventId, force](IReplayController *r) {
+  bool done = false;
+
+  QString tag = lit("replaySetEvent");
+
+  // we can't return until the event is selected, but a blocking invoke on the UI thread can cause
+  // the UI to stall. We ideally want to have at least an interactive UI and a progress bar.
+  m_Replay.AsyncInvoke(tag, [this, eventId, force, &done](IReplayController *r) {
     r->SetFrameEvent(eventId, force);
     m_CurD3D11PipelineState = r->GetD3D11PipelineState();
     m_CurD3D12PipelineState = r->GetD3D12PipelineState();
     m_CurGLPipelineState = r->GetGLPipelineState();
     m_CurVulkanPipelineState = r->GetVulkanPipelineState();
     m_CurPipelineState = &r->GetPipelineState();
+
+    done = true;
   });
 
+  // wait a short while before displaying the progress dialog (which won't show if we're already
+  // done by the time we reach it).
+  // Keep waiting if the current tag is a set event, we don't want to be popping up progress bars
+  // when the user is browsing the frame if it's going slow. If that's the case we'll just block the
+  // UI thread. Instead only pop up the progress bar if some other large task is blocking.
+  for(int i = 0; !done && (i < 100 || m_Replay.GetCurrentProcessingTag().isEmpty() ||
+                           m_Replay.GetCurrentProcessingTag() == tag);
+      i++)
+    QThread::msleep(5);
+
+  ShowProgressDialog(m_MainWindow->Widget(), tr("Please wait, working..."),
+                     [&done]() { return done; });
+
   bool updateSelectedEvent = force || prevSelectedEventID != selectedEventID;
   bool updateEvent = force || prevEventID != eventId;
 
@@ -1518,13 +1671,22 @@ void CaptureContext::SetRemoteHost(int hostIdx)
 
 bool CaptureContext::IsResourceReplaced(ResourceId id)
 {
-  return m_ReplacedResources.contains(id);
+  return m_OrigToReplacedResources.contains(id);
 }
 
-void CaptureContext::RegisterReplacement(ResourceId id)
+ResourceId CaptureContext::GetResourceReplacement(ResourceId id)
 {
-  if(!m_ReplacedResources.contains(id))
-    m_ReplacedResources.push_back(id);
+  auto it = m_OrigToReplacedResources.find(id);
+  if(it != m_OrigToReplacedResources.end())
+    return it.value();
+
+  return ResourceId();
+}
+
+void CaptureContext::RegisterReplacement(ResourceId from, ResourceId to)
+{
+  m_OrigToReplacedResources[from] = to;
+  m_ReplacedToOrigResources[to] = from;
 
   CacheResources();
 
@@ -1533,7 +1695,12 @@ void CaptureContext::RegisterReplacement(ResourceId id)
 
 void CaptureContext::UnregisterReplacement(ResourceId id)
 {
-  m_ReplacedResources.removeOne(id);
+  auto it = m_OrigToReplacedResources.find(id);
+  if(it != m_OrigToReplacedResources.end())
+  {
+    m_ReplacedToOrigResources.remove(it.value());
+    m_OrigToReplacedResources.remove(it.key());
+  }
 
   CacheResources();
 
@@ -1552,9 +1719,11 @@ void CaptureContext::RefreshUIStatus(const rdcarray<ICaptureViewer *> &exclude,
       PointerTypeRegistry::CacheShader(refl);
   }
 
-  for(ICaptureViewer *viewer : m_CaptureViewers)
+  rdcarray<ICaptureViewer *> capviewers(m_CaptureViewers);
+
+  for(ICaptureViewer *viewer : capviewers)
   {
-    if(exclude.contains(viewer))
+    if(!viewer || exclude.contains(viewer) || !m_CaptureViewers.contains(viewer))
       continue;
 
     if(updateSelectedEvent)
@@ -1813,14 +1982,14 @@ void CaptureContext::LoadEdits(const QString &data)
     ApplyShaderEdit(viewer, id, stage, shaderEncoding, flags, entryFunc, shaderBytes);
   };
 
-  auto replaceCloseCallback = [this](ICaptureContext *ctx, IShaderViewer *view, ResourceId id) {
+  auto replaceRevertCallback = [this](ICaptureContext *ctx, IShaderViewer *view, ResourceId id) {
     RevertShaderEdit(view, id);
   };
 
   for(QVariant e : editors)
   {
     ShaderViewer *edit =
-        ShaderViewer::LoadEditor(*this, e.toMap(), replaceSaveCallback, replaceCloseCallback,
+        ShaderViewer::LoadEditor(*this, e.toMap(), replaceSaveCallback, replaceRevertCallback,
                                  [this](ShaderViewer *view, bool closed) {
                                    SetModification(CaptureModifications::EditedShaders);
                                    if(closed)
@@ -1912,6 +2081,9 @@ rdcstr CaptureContext::GetResourceNameUnsuffixed(ResourceId id)
   if(id == ResourceId())
     return tr("No Resource");
 
+  if(m_ReplacedToOrigResources.contains(id))
+    return GetResourceName(m_ReplacedToOrigResources[id]);
+
   ResourceDescription *desc = GetResource(id);
 
   if(desc)
@@ -1934,7 +2106,7 @@ rdcstr CaptureContext::GetResourceName(ResourceId id)
 {
   rdcstr ret = GetResourceNameUnsuffixed(id);
 
-  if(m_ReplacedResources.contains(id))
+  if(m_OrigToReplacedResources.contains(id))
     ret += tr(" (Edited)");
 
   return ret;
@@ -1980,11 +2152,6 @@ void CaptureContext::SetResourceCustomName(ResourceId id, const rdcstr &name)
   RefreshUIStatus({}, true, true);
 }
 
-int CaptureContext::ResourceNameCacheID()
-{
-  return m_CustomNameCachedID;
-}
-
 #if defined(RENDERDOC_PLATFORM_APPLE)
 extern "C" void *makeNSViewMetalCompatible(void *handle);
 #endif
@@ -2029,9 +2196,9 @@ WindowingData CaptureContext::CreateWindowingData(QWidget *window)
 
   return CreateMacOSWindowingData(view, layer);
 
-#elif defined(RENDERDOC_PLATFORM_APPLE)
+#elif defined(RENDERDOC_PLATFORM_GGP)
 
-  WindowingData ret = {WindowingSystem::Unknown};
+  WindowingData ret = {WindowingSystem::GGP};
   return ret;
 
 #else
@@ -2299,7 +2466,7 @@ IShaderViewer *CaptureContext::EditShader(ResourceId id, ShaderStage stage,
                                           const rdcstr &entryPoint, const rdcstrpairs &files,
                                           ShaderEncoding shaderEncoding, ShaderCompileFlags flags,
                                           IShaderViewer::SaveCallback saveCallback,
-                                          IShaderViewer::CloseCallback closeCallback)
+                                          IShaderViewer::RevertCallback revertCallback)
 {
   ShaderViewer *viewer = NULL;
 
@@ -2316,16 +2483,16 @@ IShaderViewer *CaptureContext::EditShader(ResourceId id, ShaderStage stage,
         saveCallback(ctx, viewer, id, stage, shaderEncoding, flags, entryFunc, shaderBytes);
     };
 
-    auto replaceCloseCallback = [this, closeCallback](ICaptureContext *ctx, IShaderViewer *view,
-                                                      ResourceId id) {
+    auto replaceRevertCallback = [this, revertCallback](ICaptureContext *ctx, IShaderViewer *view,
+                                                        ResourceId id) {
       RevertShaderEdit(view, id);
 
-      if(closeCallback)
-        closeCallback(ctx, view, id);
+      if(revertCallback)
+        revertCallback(ctx, view, id);
     };
 
     viewer = ShaderViewer::EditShader(*this, id, stage, entryPoint, files, shaderEncoding, flags,
-                                      replaceSaveCallback, replaceCloseCallback,
+                                      replaceSaveCallback, replaceRevertCallback,
                                       [this](ShaderViewer *view, bool closed) {
                                         SetModification(CaptureModifications::EditedShaders);
                                         if(closed)
@@ -2339,7 +2506,7 @@ IShaderViewer *CaptureContext::EditShader(ResourceId id, ShaderStage stage,
   else
   {
     viewer = ShaderViewer::EditShader(*this, id, stage, entryPoint, files, shaderEncoding, flags,
-                                      saveCallback, closeCallback, NULL, m_MainWindow->Widget());
+                                      saveCallback, revertCallback, NULL, m_MainWindow->Widget());
   }
 
   return viewer;
@@ -2365,8 +2532,7 @@ void CaptureContext::ApplyShaderEdit(IShaderViewer *viewer, ResourceId id, Shade
     ResourceId from = id;
     ResourceId to;
 
-    rdctie(to, errs) =
-        r->BuildTargetShader(entryFunc.c_str(), shaderEncoding, shaderBytes, flags, stage);
+    rdctie(to, errs) = r->BuildTargetShader(entryFunc, shaderEncoding, shaderBytes, flags, stage);
 
     if(to == ResourceId())
     {
@@ -2379,7 +2545,8 @@ void CaptureContext::ApplyShaderEdit(IShaderViewer *viewer, ResourceId id, Shade
     {
       r->ReplaceResource(from, to);
 
-      GUIInvoke::call(GetMainWindow()->Widget(), [this, from]() { RegisterReplacement(from); });
+      GUIInvoke::call(GetMainWindow()->Widget(),
+                      [this, from, to]() { RegisterReplacement(from, to); });
     }
     if(ptr)
       GUIInvoke::call(ptr, [viewer, errs]() { viewer->ShowErrors(errs); });
@@ -2388,12 +2555,18 @@ void CaptureContext::ApplyShaderEdit(IShaderViewer *viewer, ResourceId id, Shade
 
 void CaptureContext::RevertShaderEdit(IShaderViewer *viewer, ResourceId id)
 {
+  QPointer<QObject> ptr(viewer->Widget());
+
   // remove the replacement on close (we could make this more sophisticated if there
   // was a place to control replaced resources/shaders).
-  Replay().AsyncInvoke([this, id](IReplayController *r) {
+  Replay().AsyncInvoke([this, viewer, id, ptr](IReplayController *r) {
     if(IsCaptureLoaded())
       r->RemoveReplacement(id);
-    GUIInvoke::call(GetMainWindow()->Widget(), [this, id] { UnregisterReplacement(id); });
+    GUIInvoke::call(GetMainWindow()->Widget(), [this, viewer, id, ptr] {
+      UnregisterReplacement(id);
+      if(ptr)
+        viewer->ShowErrors(rdcstr());
+    });
   });
 }
 
@@ -2410,6 +2583,11 @@ IShaderViewer *CaptureContext::ViewShader(const ShaderReflection *shader, Resour
   return ShaderViewer::ViewShader(*this, shader, pipeline, m_MainWindow->Widget());
 }
 
+IShaderMessageViewer *CaptureContext::ViewShaderMessages(ShaderStageMask stages)
+{
+  return new ShaderMessageViewer(*this, stages, m_MainWindow);
+}
+
 IBufferViewer *CaptureContext::ViewBuffer(uint64_t byteOffset, uint64_t byteSize, ResourceId id,
                                           const rdcstr &format)
 {
@@ -2440,7 +2618,7 @@ IConstantBufferPreviewer *CaptureContext::ViewConstantBuffer(ShaderStage stage,
   return new ConstantBufferPreviewer(*this, stage, slot, idx, m_MainWindow);
 }
 
-IPixelHistoryView *CaptureContext::ViewPixelHistory(ResourceId texID, int x, int y,
+IPixelHistoryView *CaptureContext::ViewPixelHistory(ResourceId texID, uint32_t x, uint32_t y,
                                                     const TextureDisplay &display)
 {
   return new PixelHistoryView(*this, texID, QPoint(x, y), display, m_MainWindow);
