@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -55,6 +55,8 @@ class ResourceInspector;
 class ShaderViewer;
 class MiniQtHelper;
 
+class QFileSystemWatcher;
+
 class CaptureContext : public ICaptureContext, IExtensionManager
 {
   Q_DECLARE_TR_FUNCTIONS(CaptureContext);
@@ -63,7 +65,8 @@ public:
   CaptureContext(PersistantConfig &cfg);
   ~CaptureContext();
 
-  void Begin(QString paramFilename, QString remoteHost, uint32_t remoteIdent, bool temp);
+  void Begin(QString paramFilename, QString remoteHost, uint32_t remoteIdent, bool temp,
+             QString scriptFilename);
   bool isRunning();
 
   rdcstr TempCaptureFilename(const rdcstr &appname) override;
@@ -119,7 +122,8 @@ public:
   void SetRemoteHost(int hostIndex);
   void RefreshStatus() override { SetEventID({}, m_SelectedEventID, m_EventID, true); }
   bool IsResourceReplaced(ResourceId id) override;
-  void RegisterReplacement(ResourceId id) override;
+  ResourceId GetResourceReplacement(ResourceId id) override;
+  void RegisterReplacement(ResourceId from, ResourceId to) override;
   void UnregisterReplacement(ResourceId id) override;
   void RefreshUIStatus(const rdcarray<ICaptureViewer *> &exclude, bool updateSelectedEvent,
                        bool updateEvent);
@@ -145,6 +149,7 @@ public:
   bool IsCaptureLocal() override { return m_CaptureLocal; }
   bool IsCaptureTemporary() override { return m_CaptureTemporary; }
   bool IsCaptureLoading() override { return m_LoadInProgress; }
+  ReplayStatus GetFatalError() override { return m_Replay.GetFatalError(); }
   rdcstr GetCaptureFilename() override { return m_CaptureFile; }
   CaptureModifications GetCaptureModifications() override { return m_CaptureMods; }
   const FrameDescription &FrameInfo() override { return m_FrameInfo; }
@@ -153,16 +158,13 @@ public:
   rdcarray<ShaderEncoding> TargetShaderEncodings() override { return m_TargetEncodings; }
   uint32_t CurSelectedEvent() override { return m_SelectedEventID; }
   uint32_t CurEvent() override { return m_EventID; }
-  const DrawcallDescription *CurSelectedDrawcall() override
-  {
-    return GetDrawcall(CurSelectedEvent());
-  }
-  const DrawcallDescription *CurDrawcall() override { return GetDrawcall(CurEvent()); }
-  const DrawcallDescription *GetFirstDrawcall() override { return m_FirstDrawcall; };
-  const DrawcallDescription *GetLastDrawcall() override { return m_LastDrawcall; };
+  const ActionDescription *CurSelectedAction() override { return GetAction(CurSelectedEvent()); }
+  const ActionDescription *CurAction() override { return GetAction(CurEvent()); }
+  const ActionDescription *GetFirstAction() override { return m_FirstAction; };
+  const ActionDescription *GetLastAction() override { return m_LastAction; };
   bool OpenRGPProfile(const rdcstr &filename) override;
   IRGPInterop *GetRGPInterop() override { return m_RGP; }
-  const rdcarray<DrawcallDescription> &CurDrawcalls() override { return *m_Drawcalls; }
+  const rdcarray<ActionDescription> &CurRootActions() override { return *m_Actions; }
   ResourceDescription *GetResource(ResourceId id) override { return m_Resources[id]; }
   const rdcarray<ResourceDescription> &GetResources() override { return m_ResourceList; }
   rdcstr GetResourceName(ResourceId id) override;
@@ -170,21 +172,21 @@ public:
   bool IsAutogeneratedName(ResourceId id) override;
   bool HasResourceCustomName(ResourceId id) override;
   void SetResourceCustomName(ResourceId id, const rdcstr &name) override;
-  int ResourceNameCacheID() override;
+  int32_t ResourceNameCacheID() override { return m_CustomNameCachedID; }
   TextureDescription *GetTexture(ResourceId id) override { return m_Textures[id]; }
   const rdcarray<TextureDescription> &GetTextures() override { return m_TextureList; }
   BufferDescription *GetBuffer(ResourceId id) override { return m_Buffers[id]; }
   const rdcarray<BufferDescription> &GetBuffers() override { return m_BufferList; }
-  const DrawcallDescription *GetDrawcall(uint32_t eventId) override
+  const ActionDescription *GetAction(uint32_t eventId) override
   {
-    return GetDrawcall(*m_Drawcalls, eventId);
+    return GetAction(*m_Actions, eventId);
   }
   const SDFile &GetStructuredFile() override { return *m_StructuredFile; }
   WindowingSystem CurWindowingSystem() override { return m_CurWinSystem; }
   WindowingData CreateWindowingData(QWidget *window) override;
 
   const rdcarray<DebugMessage> &DebugMessages() override { return m_DebugMessages; }
-  int UnreadMessageCount() override { return m_UnreadMessageCount; }
+  int32_t UnreadMessageCount() override { return m_UnreadMessageCount; }
   void MarkMessagesRead() override { m_UnreadMessageCount = 0; }
   void AddMessages(const rdcarray<DebugMessage> &msgs) override;
 
@@ -244,7 +246,7 @@ public:
   IShaderViewer *EditShader(ResourceId id, ShaderStage stage, const rdcstr &entryPoint,
                             const rdcstrpairs &files, ShaderEncoding shaderEncoding,
                             ShaderCompileFlags flags, IShaderViewer::SaveCallback saveCallback,
-                            IShaderViewer::CloseCallback closeCallback) override;
+                            IShaderViewer::RevertCallback revertCallback) override;
 
   void ApplyShaderEdit(IShaderViewer *viewer, ResourceId id, ShaderStage stage,
                        ShaderEncoding shaderEncoding, ShaderCompileFlags flags,
@@ -257,6 +259,8 @@ public:
 
   IShaderViewer *ViewShader(const ShaderReflection *shader, ResourceId pipeline) override;
 
+  IShaderMessageViewer *ViewShaderMessages(ShaderStageMask stages) override;
+
   IBufferViewer *ViewBuffer(uint64_t byteOffset, uint64_t byteSize, ResourceId id,
                             const rdcstr &format = "") override;
   IBufferViewer *ViewTextureAsBuffer(ResourceId id, const Subresource &sub,
@@ -264,7 +268,7 @@ public:
 
   IConstantBufferPreviewer *ViewConstantBuffer(ShaderStage stage, uint32_t slot,
                                                uint32_t idx) override;
-  IPixelHistoryView *ViewPixelHistory(ResourceId texID, int x, int y,
+  IPixelHistoryView *ViewPixelHistory(ResourceId texID, uint32_t x, uint32_t y,
                                       const TextureDisplay &display) override;
 
   QWidget *CreateBuiltinWindow(const rdcstr &objectName) override;
@@ -335,33 +339,33 @@ private:
   uint32_t m_SelectedEventID = 0;
   uint32_t m_EventID = 0;
 
-  const DrawcallDescription *GetDrawcall(const rdcarray<DrawcallDescription> &draws, uint32_t eventId)
+  const ActionDescription *GetAction(const rdcarray<ActionDescription> &actions, uint32_t eventId)
   {
-    for(const DrawcallDescription &d : draws)
+    for(const ActionDescription &a : actions)
     {
-      if(!d.children.empty())
+      if(!a.children.empty())
       {
-        const DrawcallDescription *draw = GetDrawcall(d.children, eventId);
-        if(draw != NULL)
-          return draw;
+        const ActionDescription *action = GetAction(a.children, eventId);
+        if(action != NULL)
+          return action;
       }
 
-      if(d.eventId == eventId)
-        return &d;
+      if(a.eventId == eventId)
+        return &a;
     }
 
     return NULL;
   }
 
   void setupDockWindow(QWidget *shad);
-  const rdcarray<DrawcallDescription> *m_Drawcalls;
-  rdcarray<DrawcallDescription> m_EmptyDraws;
+  const rdcarray<ActionDescription> *m_Actions;
+  rdcarray<ActionDescription> m_EmptyActions;
 
   rdcarray<ShaderEncoding> m_CustomEncodings, m_TargetEncodings;
   APIProperties m_APIProps;
   FrameDescription m_FrameInfo;
-  const DrawcallDescription *m_FirstDrawcall = NULL;
-  const DrawcallDescription *m_LastDrawcall = NULL;
+  const ActionDescription *m_FirstAction = NULL;
+  const ActionDescription *m_LastAction = NULL;
 
   IRGPInterop *m_RGP = NULL;
 
@@ -379,7 +383,10 @@ private:
   QMap<ResourceId, QString> m_CustomNames;
   int m_CustomNameCachedID = 1;
 
-  QVector<ResourceId> m_ReplacedResources;
+  // map orig replaced -> edited replacement ID
+  QMap<ResourceId, ResourceId> m_OrigToReplacedResources;
+  // reverse map, edited ID -> orig
+  QMap<ResourceId, ResourceId> m_ReplacedToOrigResources;
 
   const SDFile *m_StructuredFile = NULL;
   SDFile m_DummySDFile;
@@ -405,6 +412,8 @@ private:
 
   MiniQtHelper *m_QtHelper = NULL;
 
+  QFileSystemWatcher *m_Watcher = NULL;
+
   // Windows
   MainWindow *m_MainWindow = NULL;
   EventBrowser *m_EventBrowser = NULL;
