@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -48,6 +48,8 @@ void ReplayManager::OpenCapture(const QString &capturefile, const ReplayOptions
   if(m_Running)
     return;
 
+  m_FatalError = ReplayStatus::Succeeded;
+
   // TODO maybe we could expose this choice to the user?
   int proxyRenderer = -1;
 
@@ -82,7 +84,7 @@ void ReplayManager::DeleteCapture(const rdcstr &capture, bool local)
     if(m_Remote)
     {
       QMutexLocker autolock(&m_RemoteLock);
-      m_Remote->TakeOwnershipCapture(capture.c_str());
+      m_Remote->TakeOwnershipCapture(capture);
     }
   }
 }
@@ -136,9 +138,7 @@ void ReplayManager::ListFolder(const rdcstr &path, bool synchronous, DirectoryBr
 
   if(IsRunning() && m_Thread->isCurrentThread())
   {
-    auto lambda = [cb, path, this](IReplayController *r) {
-      cb(path, m_Remote->ListFolder(path.c_str()));
-    };
+    auto lambda = [cb, path, this](IReplayController *r) { cb(path, m_Remote->ListFolder(path)); };
 
     if(synchronous)
       BlockInvoke(lambda);
@@ -152,7 +152,7 @@ void ReplayManager::ListFolder(const rdcstr &path, bool synchronous, DirectoryBr
   // prevent pings while fetching remote FS data
   {
     QMutexLocker autolock(&m_RemoteLock);
-    contents = m_Remote->ListFolder(path.c_str());
+    contents = m_Remote->ListFolder(path);
   }
 
   cb(path, contents);
@@ -172,8 +172,7 @@ rdcstr ReplayManager::CopyCaptureToRemote(const rdcstr &localpath, QWidget *wind
 
   auto lambda = [this, localpath, &remotepath, &progress, &copied](IReplayController *r) {
     QMutexLocker autolock(&m_RemoteLock);
-    remotepath =
-        m_Remote->CopyCaptureToRemote(localpath.c_str(), [&progress](float p) { progress = p; });
+    remotepath = m_Remote->CopyCaptureToRemote(localpath, [&progress](float p) { progress = p; });
     copied = 1;
   };
 
@@ -207,8 +206,7 @@ void ReplayManager::CopyCaptureFromRemote(const rdcstr &remotepath, const rdcstr
 
   auto lambda = [this, localpath, remotepath, &progress, &copied](IReplayController *r) {
     QMutexLocker autolock(&m_RemoteLock);
-    m_Remote->CopyCaptureFromRemote(remotepath.c_str(), localpath.c_str(),
-                                    [&progress](float p) { progress = p; });
+    m_Remote->CopyCaptureFromRemote(remotepath, localpath, [&progress](float p) { progress = p; });
     copied = 1;
   };
 
@@ -240,6 +238,12 @@ float ReplayManager::GetCurrentProcessingTime()
   return m_CommandTimer.isValid() ? double(m_CommandTimer.elapsed()) / 1000.0 : 0.0;
 }
 
+QString ReplayManager::GetCurrentProcessingTag()
+{
+  QMutexLocker lock(&m_TimerLock);
+  return m_CommandTag;
+}
+
 void ReplayManager::AsyncInvoke(const rdcstr &tag, ReplayManager::InvokeCallback m)
 {
   QString qtag(tag);
@@ -294,6 +298,7 @@ void ReplayManager::CancelReplayLoop()
 void ReplayManager::CloseThread()
 {
   m_Running = false;
+  m_FatalError = ReplayStatus::Succeeded;
 
   m_RenderCondition.wakeAll();
 
@@ -389,7 +394,7 @@ void ReplayManager::ReopenCaptureFile(const QString &path)
 {
   if(!m_CaptureFile)
     m_CaptureFile = RENDERDOC_OpenCaptureFile();
-  m_CaptureFile->OpenFile(path.toUtf8().data(), "rdc", NULL);
+  m_CaptureFile->OpenFile(path, "rdc", NULL);
 }
 
 ExecuteResult ReplayManager::ExecuteAndInject(const rdcstr &exe, const rdcstr &workingDir,
@@ -402,12 +407,11 @@ ExecuteResult ReplayManager::ExecuteAndInject(const rdcstr &exe, const rdcstr &w
   if(m_Remote)
   {
     QMutexLocker autolock(&m_RemoteLock);
-    ret = m_Remote->ExecuteAndInject(exe.c_str(), workingDir.c_str(), cmdLine.c_str(), env, opts);
+    ret = m_Remote->ExecuteAndInject(exe, workingDir, cmdLine, env, opts);
   }
   else
   {
-    ret = RENDERDOC_ExecuteAndInject(exe.c_str(), workingDir.c_str(), cmdLine.c_str(), env,
-                                     capturefile.c_str(), opts, false);
+    ret = RENDERDOC_ExecuteAndInject(exe, workingDir, cmdLine, env, capturefile, opts, false);
   }
 
   return ret;
@@ -437,13 +441,13 @@ void ReplayManager::run(int proxyRenderer, const QString &capturefile, const Rep
   if(m_Remote)
   {
     rdctie(m_CreateStatus, m_Renderer) =
-        m_Remote->OpenCapture(proxyRenderer, capturefile.toUtf8().data(), opts, progress);
+        m_Remote->OpenCapture(proxyRenderer, capturefile, opts, progress);
   }
   else
   {
     m_CaptureFile = RENDERDOC_OpenCaptureFile();
 
-    m_CreateStatus = m_CaptureFile->OpenFile(capturefile.toUtf8().data(), "rdc", NULL);
+    m_CreateStatus = m_CaptureFile->OpenFile(capturefile, "rdc", NULL);
 
     if(m_CreateStatus == ReplayStatus::Succeeded)
       rdctie(m_CreateStatus, m_Renderer) = m_CaptureFile->OpenCapture(opts, progress);
@@ -486,13 +490,22 @@ void ReplayManager::run(int proxyRenderer, const QString &capturefile, const Rep
       {
         QMutexLocker lock(&m_TimerLock);
         m_CommandTimer.start();
+        m_CommandTag = cmd->tag;
       }
 
       cmd->method(m_Renderer);
 
+      ReplayStatus err = m_Renderer->GetFatalErrorStatus();
+      if(m_FatalError == ReplayStatus::Succeeded && err != ReplayStatus::Succeeded)
+      {
+        m_FatalError = err;
+        m_FatalErrorCallback();
+      }
+
       {
         QMutexLocker lock(&m_TimerLock);
         m_CommandTimer.invalidate();
+        m_CommandTag = QString();
       }
     }
 
