@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -54,7 +54,10 @@ public:
 
   bool IsRunning();
   ReplayStatus GetCreateStatus() { return m_CreateStatus; }
+  void SetFatalErrorCallback(std::function<void()> cb) { m_FatalErrorCallback = cb; }
+  ReplayStatus GetFatalError() { return m_FatalError; }
   float GetCurrentProcessingTime();
+  QString GetCurrentProcessingTag();
   // this tagged version is for cases when we might send a request - e.g. to pick a vertex or pixel
   // - and want to pre-empt it with a new request before the first has returned. Either because some
   // other work is taking a while or because we're sending requests faster than they can be
@@ -119,6 +122,7 @@ private:
 
   QMutex m_TimerLock;
   QElapsedTimer m_CommandTimer;
+  QString m_CommandTag;
 
   QMutex m_RenderLock;
   QQueue<InvokeHandle *> m_RenderQueue;
@@ -134,6 +138,8 @@ private:
   IRemoteServer *m_Remote = NULL;
   IRemoteServer *m_OrphanedRemote = NULL;
 
+  std::function<void()> m_FatalErrorCallback;
+  ReplayStatus m_FatalError = ReplayStatus::Succeeded;
   volatile bool m_Running;
   LambdaThread *m_Thread;
   ReplayStatus m_CreateStatus = ReplayStatus::Succeeded;
