@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -64,4 +64,12 @@ extern "C" void SetThreadBlocking(PyObject *global_handle, bool block)
 {
 }
 
+extern "C" void QueueDecRef(PyObject *obj)
+{
+}
+
+extern "C" void ProcessDecRefQueue()
+{
+}
+
 REPLAY_PROGRAM_MARKER()
