@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -61,7 +61,7 @@ void sharedLogOutput(QtMsgType type, const QMessageLogContext &context, const QS
     case QtFatalMsg: logtype = LogType::Fatal; break;
   }
 
-  RENDERDOC_LogMessage(logtype, "QTRD", context.file, context.line, msg.toUtf8().data());
+  RENDERDOC_LogMessage(logtype, "QTRD", context.file ? context.file : rdcstr(), context.line, msg);
 }
 
 static QString tr(const char *string)
@@ -86,6 +86,11 @@ int main(int argc, char *argv[])
 
   qInstallMessageHandler(sharedLogOutput);
 
+  // there seems to be a persistent crash in QWidgetPrivate::subtractOpaqueSiblings where a widget
+  // has no parent but is not a window. Try to work around it by setting this env var, as it's only
+  // an optimisation
+  qputenv("QT_NO_SUBTRACTOPAQUESIBLINGS", lit("1").toUtf8());
+
   qInfo() << "QRenderDoc initialising.";
 
   if(IsRunningAsAdmin())
@@ -129,7 +134,7 @@ int main(int argc, char *argv[])
 
     if(errors)
     {
-      RENDERDOC_LogMessage(LogType::Error, "EXTN", __FILE__, __LINE__, errorLog.c_str());
+      RENDERDOC_LogMessage(LogType::Error, "EXTN", __FILE__, __LINE__, errorLog);
       fputs("Found errors in python bindings. Please fix!\n", logOut);
       fputs(errorLog.c_str(), logOut);
       return 1;
@@ -167,6 +172,11 @@ int main(int argc, char *argv[])
                             lit("filename.py"));
   parser.addOption(python);
 
+  QCommandLineOption uiscript({lit("ui-python"), lit("ui-script"), lit("ui-py")},
+                              tr("Run a python script after opening the main UI."),
+                              lit("filename.py"));
+  parser.addOption(uiscript);
+
   // secret non-described options
   QCommandLineOption installLayer(lit("install_vulkan_layer"), QString(), lit("root_or_not"));
   hideOption(installLayer);
@@ -259,7 +269,7 @@ int main(int argc, char *argv[])
     else
     {
       // no port specified, find the first open port.
-      ident = RENDERDOC_EnumerateRemoteTargets(host.toLocal8Bit().data(), ident);
+      ident = RENDERDOC_EnumerateRemoteTargets(host, ident);
       ok = (ident != 0);
     }
 
@@ -283,6 +293,10 @@ int main(int argc, char *argv[])
   if(parser.isSet(crashReport))
     crashReportPath = parser.value(crashReport);
 
+  QString uiscriptFile;
+  if(parser.isSet(uiscript))
+    uiscriptFile = parser.value(uiscript);
+
   QStringList pyscripts = parser.values(python);
 
   // load the first filename in the positional arguments.
@@ -314,7 +328,7 @@ int main(int argc, char *argv[])
         dir.mkpath(configPath);
     }
 
-    QString configFilename = configFilePath(lit("UI.config"));
+    QString configFilename = ConfigFilePath(lit("UI.config"));
 
     if(!config.Load(configFilename))
     {
@@ -515,7 +529,7 @@ int main(int argc, char *argv[])
 
       if(!pythonExited)
       {
-        ctx.Begin(filename, remoteHost, remoteIdent, temp);
+        ctx.Begin(filename, remoteHost, remoteIdent, temp, uiscriptFile);
 
         while(ctx.isRunning())
         {
