@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -49,7 +49,6 @@ CustomPaintWidget::CustomPaintWidget(QWidget *parent) : QWidget(parent)
   m_Tag = QFormatStr("custompaint%1").arg((uintptr_t) this);
 
   setAttribute(Qt::WA_OpaquePaintEvent);
-  setAttribute(Qt::WA_PaintOnScreen);
 
   m_Dark = Formatter::DarkCheckerColor();
   m_Light = Formatter::LightCheckerColor();
@@ -97,7 +96,12 @@ void CustomPaintWidget::OnSelectedEventChanged(uint32_t eventId)
 
 void CustomPaintWidget::OnEventChanged(uint32_t eventId)
 {
-  // nothing, we only care about capture loaded/closed events
+  // if we've encountered a fatal error recreate the widget and take over painting
+  if(m_Rendering && m_Ctx && m_Ctx->GetFatalError() != ReplayStatus::Succeeded)
+  {
+    RecreateInternalWidget();
+    update();
+  }
 }
 
 void CustomPaintWidget::update()
@@ -131,8 +135,9 @@ void CustomPaintWidget::RecreateInternalWidget()
     return;
   }
 
-  // if no capture is loaded, we're not rendering anymore.
-  m_Rendering = m_Rendering && m_Ctx && m_Ctx->IsCaptureLoaded();
+  // if no capture is loaded, or we've encountered a fatal error, we're not rendering anymore.
+  m_Rendering = m_Rendering && m_Ctx && m_Ctx->IsCaptureLoaded() &&
+                m_Ctx->GetFatalError() == ReplayStatus::Succeeded;
 
   // we need to recreate the widget if it's not matching out rendering state.
   if(m_Internal == NULL || m_Rendering != m_Internal->IsRendering())
@@ -156,11 +161,11 @@ void CustomPaintWidget::changeEvent(QEvent *event)
 
 void CustomPaintWidget::renderInternal(QPaintEvent *e)
 {
-  if(m_Ctx && m_Output)
+  if(m_Ctx && m_Output && m_Ctx->IsCaptureLoaded())
   {
     QPointer<CustomPaintWidget> me(this);
     m_Ctx->Replay().AsyncInvoke(m_Tag, [me](IReplayController *r) {
-      if(me && me->m_Output)
+      if(me && me->m_Output && me->m_Ctx->IsCaptureLoaded())
         me->m_Output->Display();
     });
   }
