@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,11 +23,13 @@
  ******************************************************************************/
 
 #include "RDHeaderView.h"
+#include <QAbstractScrollArea>
 #include <QLabel>
 #include <QMouseEvent>
 #include <QPainter>
 #include <QPixmap>
 #include <QPointer>
+#include <QScrollBar>
 #include <QTreeView>
 #include "Code/QRDUtils.h"
 
@@ -109,6 +111,8 @@ void RDHeaderView::reset()
 {
   if(m_customSizing)
     cacheSections();
+
+  QHeaderView::reset();
 }
 
 void RDHeaderView::cacheSections()
@@ -245,7 +249,10 @@ int RDHeaderView::visualIndexAt(int position) const
 
 int RDHeaderView::logicalIndexAt(int position) const
 {
-  return visualIndexAt(position);
+  if(m_customSizing)
+    return visualIndexAt(position);
+
+  return QHeaderView::logicalIndexAt(position);
 }
 
 int RDHeaderView::count() const
@@ -497,6 +504,13 @@ void RDHeaderView::setColumnStretchHints(const QList<int> &hints)
   resizeSectionsWithHints();
 }
 
+void RDHeaderView::setPinnedColumns(int numColumns, QAbstractScrollArea *scroll)
+{
+  m_pinnedColumns = numColumns;
+  QObject::connect(scroll->horizontalScrollBar(), &QScrollBar::valueChanged,
+                   [this]() { viewport()->update(); });
+}
+
 void RDHeaderView::setRootIndex(const QModelIndex &index)
 {
   QHeaderView::setRootIndex(index);
@@ -561,6 +575,8 @@ void RDHeaderView::mousePressEvent(QMouseEvent *event)
       paintSection(&painter, QRect(QPoint(0, 0), m_sectionPreview->size()), idx);
       painter.end();
 
+      m_cursorPos = QCursor::pos().x();
+
       m_sectionPreview->setPixmap(preview);
 
       m_sectionPreviewOffset = mousePos - secPos;
@@ -568,6 +584,7 @@ void RDHeaderView::mousePressEvent(QMouseEvent *event)
       m_sectionPreview->move(mousePos - m_sectionPreviewOffset, 0);
       m_sectionPreview->show();
 
+      QHeaderView::mousePressEvent(event);
       return;
     }
   }
@@ -732,6 +749,8 @@ void RDHeaderView::mouseReleaseEvent(QMouseEvent *event)
     int mousePos = event->x();
     int idx = logicalIndexAt(mousePos);
 
+    m_sectionPreview->hide();
+
     if(idx >= 0)
     {
       int secSize = sectionSize(idx);
@@ -760,21 +779,17 @@ void RDHeaderView::mouseReleaseEvent(QMouseEvent *event)
           else
             moveSection(srcSection, dstSection);
         }
+
+        return;
       }
     }
-
-    m_sectionPreview->hide();
   }
 
   m_movingSection = -1;
 
   if(m_customSizing)
-  {
     m_resizeState = qMakePair(NoResize, -1);
 
-    return QAbstractItemView::mouseReleaseEvent(event);
-  }
-
   QHeaderView::mouseReleaseEvent(event);
 }
 
