@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -26,14 +26,18 @@
 #include <QApplication>
 #include <QClipboard>
 #include <QContextMenuEvent>
+#include <QDesktopWidget>
+#include <QHeaderView>
 #include <QLabel>
 #include <QMenu>
 #include <QMouseEvent>
 #include <QPainter>
 #include <QProxyStyle>
+#include <QScrollBar>
+#include <QStack>
 #include <QStylePainter>
-#include <QToolTip>
 #include <QWheelEvent>
+#include "Code/QRDUtils.h"
 #include "Code/Resources.h"
 
 static int GetDepth(const QAbstractItemModel *model, const QModelIndex &idx)
@@ -81,16 +85,26 @@ static bool CompareModelIndex(const QModelIndex &a, const QModelIndex &b)
   return CompareModelIndex(ap, bp);
 }
 
-RDTreeViewDelegate::RDTreeViewDelegate(RDTreeView *view) : ForwardingDelegate(view), m_View(view)
+RDTreeViewDelegate::RDTreeViewDelegate(RDTreeView *view) : RichTextViewDelegate(view), m_View(view)
 {
 }
 
+void RDTreeViewDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
+                               const QModelIndex &index) const
+{
+  return RichTextViewDelegate::paint(painter, option, index);
+}
+
 QSize RDTreeViewDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
 {
-  QSize ret = ForwardingDelegate::sizeHint(option, index);
+  QSize ret = RichTextViewDelegate::sizeHint(option, index);
+
+  int minHeight = option.fontMetrics.height();
+  if(!m_View->ignoreIconSize())
+    minHeight = qMax(option.decorationSize.height(), minHeight);
 
   if(m_View->ignoreIconSize())
-    ret.setHeight(qMax(option.decorationSize.height() + 2, ret.height()));
+    ret.setHeight(qMax(qMax(option.decorationSize.height(), minHeight) + 2, ret.height()));
 
   // expand a pixel for the grid lines
   if(m_View->visibleGridLines())
@@ -98,9 +112,6 @@ QSize RDTreeViewDelegate::sizeHint(const QStyleOptionViewItem &option, const QMo
 
   // ensure we have at least the margin on top of font size. If the style applied more, don't add to
   // it.
-  int minHeight = option.fontMetrics.height();
-  if(!m_View->ignoreIconSize())
-    minHeight = qMax(option.decorationSize.height(), minHeight);
   ret.setHeight(qMax(ret.height(), minHeight + m_View->verticalItemMargin()));
 
   return ret;
@@ -122,6 +133,23 @@ RDTipLabel::RDTipLabel(QWidget *listener) : QLabel(NULL), mouseListener(listener
   setWindowOpacity(opacity / 255.0);
 }
 
+QSize RDTipLabel::configureTip(QWidget *, QModelIndex, QString text)
+{
+  setText(text);
+  return minimumSizeHint();
+}
+
+void RDTipLabel::showTip(QPoint pos)
+{
+  move(pos);
+  show();
+}
+
+bool RDTipLabel::forceTip(QWidget *widget, QModelIndex idx)
+{
+  return false;
+}
+
 void RDTipLabel::paintEvent(QPaintEvent *ev)
 {
   QStylePainter p(this);
@@ -177,23 +205,86 @@ RDTreeView::RDTreeView(QWidget *parent) : QTreeView(parent)
   m_delegate = new RDTreeViewDelegate(this);
   QTreeView::setItemDelegate(m_delegate);
 
-  m_ElidedTooltip = new RDTipLabel(viewport());
-  m_ElidedTooltip->hide();
+  m_TooltipLabel = new RDTipLabel(viewport());
+  m_TooltipLabel->hide();
+  m_CurrentTooltipElided = false;
+
+  m_Tooltip = m_TooltipLabel;
 }
 
 RDTreeView::~RDTreeView()
 {
   setModel(NULL);
 
-  delete m_ElidedTooltip;
+  delete m_TooltipLabel;
 }
 
 void RDTreeView::mouseMoveEvent(QMouseEvent *e)
 {
-  if(m_ElidedTooltip->isVisible() && !m_ElidedTooltip->geometry().contains(QCursor::pos()))
-    m_ElidedTooltip->hide();
+  QModelIndex oldHoverIndex = m_currentHoverIndex;
+
+  if(m_CurrentTooltipElided && m_TooltipLabel->isVisible() &&
+     !m_TooltipLabel->geometry().contains(QCursor::pos()))
+    m_Tooltip->hideTip();
 
   m_currentHoverIndex = indexAt(e->pos());
+
+  if(m_delegate->linkHover(e, font(), m_currentHoverIndex))
+  {
+    if(cursor().shape() != Qt::PointingHandCursor)
+    {
+      viewport()->update(visualRect(m_currentHoverIndex));
+      setCursor(QCursor(Qt::PointingHandCursor));
+    }
+  }
+  else if(cursor().shape() == Qt::PointingHandCursor)
+  {
+    viewport()->update(visualRect(m_currentHoverIndex));
+    unsetCursor();
+  }
+
+  if(oldHoverIndex != m_currentHoverIndex)
+  {
+    if(m_instantTooltips)
+    {
+      m_Tooltip->hideTip();
+
+      if(m_currentHoverIndex.isValid())
+      {
+        QString tooltip = m_currentHoverIndex.data(Qt::ToolTipRole).toString();
+
+        if(!tooltip.isEmpty() || m_Tooltip->forceTip(this, m_currentHoverIndex))
+        {
+          // We don't use QToolTip since we have a custom tooltip for showing elided results, and we
+          // use that for consistency. This also makes it easier to slot in a custom tooltip widget
+          // externally.
+          QPoint p = QCursor::pos();
+
+          // estimate, as this is not easily queryable
+          const QPoint cursorSize(16, 16);
+          const QRect screenAvailGeom = QApplication::desktop()->availableGeometry(p);
+
+          // start with the tooltip placed bottom-right of the cursor, as the default
+          QRect tooltipRect;
+          tooltipRect.setTopLeft(p + cursorSize);
+          tooltipRect.setSize(m_Tooltip->configureTip(this, m_currentHoverIndex, tooltip));
+
+          // clip by the available geometry in x
+          if(tooltipRect.right() > screenAvailGeom.right())
+            tooltipRect.moveRight(screenAvailGeom.right());
+
+          // if we'd go out of bounds in y, place the tooltip above the cursor. Don't just clip like
+          // in x, because that could place the tooltip over the cursor.
+          if(tooltipRect.bottom() > screenAvailGeom.bottom())
+            tooltipRect.moveBottom(p.y() - cursorSize.y());
+
+          m_Tooltip->showTip(tooltipRect.topLeft());
+          m_CurrentTooltipElided = false;
+        }
+      }
+    }
+  }
+
   QTreeView::mouseMoveEvent(e);
 }
 
@@ -205,8 +296,15 @@ void RDTreeView::wheelEvent(QWheelEvent *e)
 
 void RDTreeView::leaveEvent(QEvent *e)
 {
-  if(m_ElidedTooltip->isVisible() && !m_ElidedTooltip->geometry().contains(QCursor::pos()))
-    m_ElidedTooltip->hide();
+  if(m_CurrentTooltipElided)
+  {
+    if(m_TooltipLabel->isVisible() && !m_TooltipLabel->geometry().contains(QCursor::pos()))
+      m_Tooltip->hideTip();
+  }
+  else
+  {
+    m_Tooltip->hideTip();
+  }
 
   m_currentHoverIndex = QModelIndex();
 
@@ -251,6 +349,8 @@ void RDTreeView::contextMenuEvent(QContextMenuEvent *event)
   expandAllAction.setIcon(Icons::arrow_out());
   collapseAllAction.setIcon(Icons::arrow_in());
 
+  copy.setIcon(Icons::copy());
+
   expandAllAction.setEnabled(index.isValid() && model()->rowCount(index) > 0);
   collapseAllAction.setEnabled(index.isValid() && model()->rowCount(index) > 0);
 
@@ -258,73 +358,109 @@ void RDTreeView::contextMenuEvent(QContextMenuEvent *event)
 
   QObject::connect(&collapseAllAction, &QAction::triggered, [this, index]() { collapseAll(index); });
 
-  QObject::connect(&copy, &QAction::triggered, [this, index, pos]() {
-    bool clearsel = false;
-    if(selectionModel()->selectedRows().empty())
-    {
-      setSelection(QRect(pos, QSize(1, 1)), selectionCommand(index));
-      clearsel = true;
-    }
-    copySelection();
-    if(clearsel)
-      selectionModel()->clear();
-  });
+  QObject::connect(&copy, &QAction::triggered, [this, index, pos]() { copyIndex(pos, index); });
 
   RDDialog::show(&contextMenu, viewport()->mapToGlobal(pos));
 }
 
-void RDTreeView::expandAll(QModelIndex index)
+void RDTreeView::copyIndex(QPoint pos, QModelIndex index)
 {
+  bool clearsel = false;
+  if(selectionModel()->selectedRows().empty())
+  {
+    setSelection(QRect(pos, QSize(1, 1)), selectionCommand(index));
+    clearsel = true;
+  }
+  copySelection();
+  if(clearsel)
+    selectionModel()->clear();
+}
+
+void RDTreeView::expandAllInternal(QModelIndex index)
+{
+  int rows = model()->rowCount(index);
+
+  if(rows == 0)
+    return;
+
   expand(index);
 
-  for(int r = 0, rows = model()->rowCount(index); r < rows; r++)
+  for(int r = 0; r < rows; r++)
     expandAll(model()->index(r, 0, index));
 }
 
+void RDTreeView::collapseAllInternal(QModelIndex index)
+{
+  int rows = model()->rowCount(index);
+
+  if(rows == 0)
+    return;
+
+  collapse(index);
+
+  for(int r = 0; r < rows; r++)
+    collapseAll(model()->index(r, 0, index));
+}
+
+void RDTreeView::expandAll(QModelIndex index)
+{
+  setUpdatesEnabled(false);
+  expandAllInternal(index);
+  setUpdatesEnabled(true);
+}
+
 void RDTreeView::collapseAll(QModelIndex index)
 {
-  collapse(index);
-
-  for(int r = 0, rows = model()->rowCount(index); r < rows; r++)
-    collapseAll(model()->index(r, 0, index));
+  setUpdatesEnabled(false);
+  collapseAllInternal(index);
+  setUpdatesEnabled(true);
 }
 
 bool RDTreeView::viewportEvent(QEvent *event)
 {
-  if(m_TooltipElidedItems && event->type() == QEvent::ToolTip)
+  if(event->type() == QEvent::ToolTip)
   {
-    QHelpEvent *he = (QHelpEvent *)event;
-    QModelIndex index = indexAt(he->pos());
+    // if we're doing instant tooltips this is all handled in the mousemove handler, don't do
+    // anything here
+    if(m_instantTooltips)
+      return true;
 
-    QAbstractItemDelegate *delegate = m_userDelegate;
-
-    if(!delegate)
-      delegate = QTreeView::itemDelegate(index);
-
-    if(delegate)
+    if(m_TooltipElidedItems)
     {
-      QStyleOptionViewItem option;
-      option.initFrom(this);
-      option.rect = visualRect(index);
+      QHelpEvent *he = (QHelpEvent *)event;
+      QModelIndex index = indexAt(he->pos());
 
-      // delegates get first dibs at processing the event
-      bool ret = delegate->helpEvent(he, this, option, index);
+      QAbstractItemDelegate *delegate = m_userDelegate;
 
-      if(ret)
-        return true;
+      if(!delegate)
+        delegate = QTreeView::itemDelegate(index);
 
-      QSize desiredSize = delegate->sizeHint(option, index);
-
-      if(desiredSize.width() > option.rect.width())
+      if(delegate)
       {
-        const QString fullText = index.data(Qt::DisplayRole).toString();
-        if(!fullText.isEmpty())
+        QStyleOptionViewItem option;
+        option.initFrom(this);
+        option.rect = visualRect(index);
+
+        // delegates get first dibs at processing the event
+        bool ret = delegate->helpEvent(he, this, option, index);
+
+        if(ret)
+          return true;
+
+        QSize desiredSize = delegate->sizeHint(option, index);
+
+        if(desiredSize.width() > option.rect.width())
         {
-          // need to use a custom label tooltip since the QToolTip freaks out as we're placing it
-          // underneath the cursor instead of next to it (so that the tooltip lines up over the row)
-          m_ElidedTooltip->move(viewport()->mapToGlobal(option.rect.topLeft()));
-          m_ElidedTooltip->setText(fullText);
-          m_ElidedTooltip->show();
+          const QString fullText = index.data(Qt::DisplayRole).toString();
+          if(!fullText.isEmpty())
+          {
+            // need to use a custom label tooltip since the QToolTip freaks out as we're placing it
+            // underneath the cursor instead of next to it (so that the tooltip lines up over the
+            // row)
+            m_Tooltip->configureTip(this, index, fullText);
+            m_Tooltip->showTip(viewport()->mapToGlobal(option.rect.topLeft()));
+            m_CurrentTooltipElided = true;
+          }
         }
       }
     }
@@ -349,8 +485,18 @@ void RDTreeView::setModel(QAbstractItemModel *model)
   QAbstractItemModel *old = this->model();
 
   if(old)
+  {
     QObject::disconnect(old, &QAbstractItemModel::modelAboutToBeReset, this,
                         &RDTreeView::modelAboutToBeReset);
+    QObject::disconnect(old, &QAbstractItemModel::rowsAboutToBeRemoved, this,
+                        &RDTreeView::rowsAboutToBeRemoved);
+    QObject::disconnect(old, &QAbstractItemModel::columnsAboutToBeRemoved, this,
+                        &RDTreeView::columnsAboutToBeRemoved);
+    QObject::disconnect(old, &QAbstractItemModel::rowsAboutToBeMoved, this,
+                        &RDTreeView::rowsAboutToBeMoved);
+    QObject::disconnect(old, &QAbstractItemModel::columnsAboutToBeMoved, this,
+                        &RDTreeView::columnsAboutToBeMoved);
+  }
 
   QTreeView::setModel(model);
 
@@ -358,6 +504,14 @@ void RDTreeView::setModel(QAbstractItemModel *model)
   {
     QObject::connect(model, &QAbstractItemModel::modelAboutToBeReset, this,
                      &RDTreeView::modelAboutToBeReset);
+    QObject::connect(model, &QAbstractItemModel::rowsAboutToBeRemoved, this,
+                     &RDTreeView::rowsAboutToBeRemoved);
+    QObject::connect(model, &QAbstractItemModel::columnsAboutToBeRemoved, this,
+                     &RDTreeView::columnsAboutToBeRemoved);
+    QObject::connect(model, &QAbstractItemModel::rowsAboutToBeMoved, this,
+                     &RDTreeView::rowsAboutToBeMoved);
+    QObject::connect(model, &QAbstractItemModel::columnsAboutToBeMoved, this,
+                     &RDTreeView::columnsAboutToBeMoved);
   }
 }
 
@@ -366,12 +520,34 @@ void RDTreeView::modelAboutToBeReset()
   m_currentHoverIndex = QModelIndex();
 }
 
-void RDTreeView::saveExpansion(RDTreeViewExpansionState &state, const ExpansionKeyGen &keygen)
+void RDTreeView::rowsAboutToBeRemoved(const QModelIndex &parent, int first, int last)
 {
-  state.clear();
+  m_currentHoverIndex = QModelIndex();
+  QTreeView::rowsAboutToBeRemoved(parent, first, last);
+}
 
+void RDTreeView::columnsAboutToBeRemoved(const QModelIndex &parent, int first, int last)
+{
+  m_currentHoverIndex = QModelIndex();
+}
+
+void RDTreeView::rowsAboutToBeMoved(const QModelIndex &sourceParent, int sourceStart, int sourceEnd,
+                                    const QModelIndex &destinationParent, int destinationRow)
+{
+  m_currentHoverIndex = QModelIndex();
+}
+
+void RDTreeView::columnsAboutToBeMoved(const QModelIndex &sourceParent, int sourceStart,
+                                       int sourceEnd, const QModelIndex &destinationParent,
+                                       int destinationColumn)
+{
+  m_currentHoverIndex = QModelIndex();
+}
+
+void RDTreeView::updateExpansion(RDTreeViewExpansionState &state, const ExpansionKeyGen &keygen)
+{
   for(int i = 0; i < model()->rowCount(); i++)
-    saveExpansionFromRow(state, model()->index(i, 0), 0, keygen);
+    updateExpansionFromRow(state, model()->index(i, 0), 0, keygen);
 }
 
 void RDTreeView::applyExpansion(const RDTreeViewExpansionState &state, const ExpansionKeyGen &keygen)
@@ -428,6 +604,8 @@ void RDTreeView::copySelection()
 
     int depth = GetDepth(model(), idx);
 
+    QString line;
+
     for(int i = 0; i < colCount; i++)
     {
       QString format = i == 0 ? QFormatStr("%1") : QFormatStr(" %1");
@@ -438,22 +616,27 @@ void RDTreeView::copySelection()
       if(i == 0)
         text.prepend(QString((depth - minDepth) * 2, QLatin1Char(' ')));
 
-      clipData += format.arg(text, -widths[i]);
+      line += format.arg(text, -widths[i]);
     }
 
-    clipData += lit("\n");
+    clipData += line.trimmed() + lit("\n");
   }
 
   QClipboard *clipboard = QApplication::clipboard();
-  clipboard->setText(clipData.trimmed());
+  clipboard->setText(clipData);
 }
 
-void RDTreeView::saveExpansionFromRow(RDTreeViewExpansionState &state, QModelIndex idx, uint seed,
-                                      const ExpansionKeyGen &keygen)
+void RDTreeView::updateExpansionFromRow(RDTreeViewExpansionState &state, QModelIndex idx, uint seed,
+                                        const ExpansionKeyGen &keygen)
 {
   if(!idx.isValid())
     return;
 
+  int rowcount = model()->rowCount(idx);
+
+  if(rowcount == 0)
+    return;
+
   uint key = keygen(idx, seed);
   if(isExpanded(idx))
   {
@@ -462,8 +645,12 @@ void RDTreeView::saveExpansionFromRow(RDTreeViewExpansionState &state, QModelInd
     // only recurse to children if this one is expanded - forget expansion state under collapsed
     // branches. Technically we're losing information here but it allows us to skip a full expensive
     // search
-    for(int i = 0; i < model()->rowCount(idx); i++)
-      saveExpansionFromRow(state, model()->index(i, 0, idx), seed, keygen);
+    for(int i = 0; i < rowcount; i++)
+      updateExpansionFromRow(state, model()->index(i, 0, idx), seed, keygen);
+  }
+  else
+  {
+    state.remove(key);
   }
 }
 
@@ -535,8 +722,33 @@ void RDTreeView::drawRow(QPainter *painter, const QStyleOptionViewItem &options,
   }
 }
 
-void RDTreeView::fillBranchesRect(QPainter *painter, const QRect &rect, const QModelIndex &index) const
+void RDTreeView::drawBranches(QPainter *painter, const QRect &rect, const QModelIndex &index) const
 {
+  // we do our own custom branch rendering to ensure the backgrounds for the +/- markers are filled
+  // (as otherwise they don't show up well over selection or background fills) as well as to draw
+  // any vertical branch colors.
+
+  // start at the left-most side of the rect
+  QRect branchRect(rect.left(), rect.top(), indentation(), rect.height());
+
+  // first draw the coloured lines - we're only interested in parents for this, so push all the
+  // parents onto a stack
+  QStack<QModelIndex> parents;
+
+  QModelIndex parent = index.parent();
+
+  while(parent.isValid())
+  {
+    parents.push(parent);
+    parent = parent.parent();
+  }
+
+  // fill in the background behind the lines for the whole row, since by default it doesn't show up
+  // behind the tree lines.
+
+  QRect allLinesRect(rect.left(), rect.top(),
+                     (parents.count() + (rootIsDecorated() ? 1 : 0)) * indentation(), rect.height());
+
   QStyleOptionViewItem opt;
   opt.initFrom(this);
   if(selectionModel()->isSelected(index))
@@ -563,12 +775,13 @@ void RDTreeView::fillBranchesRect(QPainter *painter, const QRect &rect, const QM
   opt.rect.setWidth(depth * indentation());
   opt.showDecorationSelected = true;
   style()->drawPrimitive(QStyle::PE_PanelItemViewItem, &opt, painter, this);
-}
 
-void RDTreeView::drawBranches(QPainter *painter, const QRect &rect, const QModelIndex &index) const
-{
-  if(m_fillBranchRect)
-    fillBranchesRect(painter, rect, index);
+  QBrush back = index.data(Qt::BackgroundRole).value<QBrush>();
+
+  if(!selectionModel()->isSelected(index) && back.style() != Qt::NoBrush)
+  {
+    painter->fillRect(allLinesRect, back);
+  }
 
   if(m_VisibleBranches)
   {
@@ -587,17 +800,108 @@ void RDTreeView::drawBranches(QPainter *painter, const QRect &rect, const QModel
     if(model()->rowCount(index) == 0)
       return;
 
-    QStyleOptionViewItem opt = viewOptions();
+    QStyleOptionViewItem branchopt = viewOptions();
 
-    opt.rect = primitive;
+    branchopt.rect = primitive;
 
     // unfortunately QStyle::State_Children doesn't render ONLY the
     // open-toggle-button, but the vertical line upwards to a previous sibling.
     // For consistency, draw one downwards too.
-    opt.state = QStyle::State_Children | QStyle::State_Sibling;
+    branchopt.state = QStyle::State_Children | QStyle::State_Sibling;
     if(isExpanded(index))
-      opt.state |= QStyle::State_Open;
+      branchopt.state |= QStyle::State_Open;
 
-    style()->drawPrimitive(QStyle::PE_IndicatorBranch, &opt, painter, this);
+    style()->drawPrimitive(QStyle::PE_IndicatorBranch, &branchopt, painter, this);
   }
+
+  // we now iterate from the top-most parent down, moving in from the left
+  // we draw this after calling into drawBranches() so we paint on top of the built-in lines
+  QPen oldPen = painter->pen();
+  while(!parents.isEmpty())
+  {
+    parent = parents.pop();
+
+    back = parent.data(RDTreeView::TreeLineColorRole).value<QBrush>();
+
+    if(back.style() != Qt::NoBrush)
+    {
+      // draw a centred pen vertically down the middle of branchRect
+      painter->setPen(QPen(QBrush(back), m_treeColorLineWidth));
+
+      QPoint topCentre = QRect(branchRect).center();
+      QPoint bottomCentre = topCentre;
+
+      topCentre.setY(branchRect.top());
+      bottomCentre.setY(branchRect.bottom());
+
+      painter->drawLine(topCentre, bottomCentre);
+    }
+
+    branchRect.moveLeft(branchRect.left() + indentation());
+  }
+  painter->setPen(oldPen);
+}
+
+QModelIndex RDTreeView::moveCursor(CursorAction cursorAction, Qt::KeyboardModifiers modifiers)
+{
+  // Qt's handling for MoveLeft is a little broken when scrollbars are in use, so we customise it
+  // do almost the same thing but with a fix
+  if(cursorAction == QAbstractItemView::MoveLeft)
+  {
+    // The default MoveRight is fine. It does in order:
+    // 1. if the current item is expandable but not expanded, it expands it.
+    // 2. if SH_ItemView_ArrowKeysNavigateIntoChildren is enabled it moves to the first child of the
+    //    current item if there is one.
+    // 3. finally it tries to scroll right, either by selecting the next column or just moving the
+    //    scrollbar.
+    //
+    // That's all good, but MoveLeft is not symmetric. Meaning it will do this:
+    // 1. if the current item is expandable and expanded, collapse it, *but only if the scrollbar is
+    //    all the way to the left*.
+    // 2. if SH_ItemView_ArrowKeysNavigateIntoChildren is enabled it moves to the current item's
+    //    parent.
+    // 3. finally it tries to scroll left if it can't do that.
+    //
+    // The problem here is that because scrolling left is still the last-resort icon, pressing right
+    // to expand an item and then perhaps scrolling right is not "undone" by pressing left, since
+    // we've now scrolled so the collapse doesn't happen and instead we jump to the parent node.
+    //
+    // To fix this, we scroll first, then handle the other two cases
+
+    QModelIndex current = currentIndex();
+
+    if(selectionBehavior() == QAbstractItemView::SelectItems ||
+       selectionBehavior() == QAbstractItemView::SelectColumns)
+    {
+      int col = header()->visualIndex(current.column());
+      // move left one
+      col--;
+
+      // keep moving if the column is hiden
+      while(col >= 0 && isColumnHidden(header()->logicalIndex(col)))
+        col--;
+
+      // if we landed on a valid column (we may have gone negative if we were already on the first
+      // column) return it
+      if(col >= 0)
+      {
+        QModelIndex sel = current.sibling(current.row(), header()->logicalIndex(col));
+        if(sel.isValid())
+          return sel;
+      }
+    }
+
+    // if we didn't scroll left above by selecting an index, and the scrollbar is still not
+    // minimised, scroll it left now.
+    QScrollBar *scroll = horizontalScrollBar();
+    if(scroll->value() > scroll->minimum())
+    {
+      scroll->setValue(scroll->value() - scroll->singleStep());
+      return current;
+    }
+
+    // otherwise we can use the default behaviour
+  }
+
+  return QTreeView::moveCursor(cursorAction, modifiers);
 }
