@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,15 +24,12 @@
 
 #include "RDTreeWidget.h"
 #include <QApplication>
-#include <QClipboard>
 #include <QColor>
 #include <QDebug>
 #include <QHeaderView>
 #include <QMouseEvent>
 #include <QPainter>
 #include <QPen>
-#include <QStack>
-#include <QToolTip>
 #include "Code/Interface/QRDInterface.h"
 #include "Code/QRDUtils.h"
 #include "Code/Resources.h"
@@ -201,8 +198,7 @@ public:
     }
     else if(role == Qt::BackgroundRole)
     {
-      // item's background color takes priority but only if not selected
-      if(item->m_back != QBrush() && !widget->selectionModel()->isSelected(index))
+      if(item->m_back.style() != Qt::NoBrush)
         return item->m_back;
 
       // otherwise if we're hover-highlighting, use the highlight color at 20% opacity
@@ -216,6 +212,11 @@ public:
       // otherwise, no special background
       return QVariant();
     }
+    else if(role == RDTreeView::TreeLineColorRole)
+    {
+      if(item->m_treeCol.isValid())
+        return QBrush(item->m_treeCol);
+    }
 
     return item->data(index.column(), role);
   }
@@ -256,7 +257,7 @@ public:
       item->m_fore = value.value<QBrush>();
       ret = true;
     }
-    else if(role == Qt::ToolTipRole && !widget->m_instantTooltips)
+    else if(role == Qt::ToolTipRole)
     {
       item->m_tooltip = value.toString();
       ret = true;
@@ -308,15 +309,51 @@ void RDTreeWidgetItem::checkForResourceId(int col)
 
 void RDTreeWidgetItem::sort(int column, Qt::SortOrder order)
 {
-  std::sort(m_children.begin(), m_children.end(),
-            [column, order](const RDTreeWidgetItem *a, const RDTreeWidgetItem *b) {
-              QVariant va = a->data(column, Qt::DisplayRole);
-              QVariant vb = b->data(column, Qt::DisplayRole);
+  ICaptureContext *ctx = getCaptureContext(m_widget);
 
-              if(order == Qt::AscendingOrder)
-                return va < vb;
-              return va > vb;
-            });
+  std::stable_sort(m_children.begin(), m_children.end(),
+                   [ctx, column, order](const RDTreeWidgetItem *a, const RDTreeWidgetItem *b) {
+
+                     if(a->treeWidget()->m_SortComparison)
+                       return a->treeWidget()->m_SortComparison(column, order, a, b);
+
+                     QVariant va = a->data(column, Qt::DisplayRole);
+                     QVariant vb = b->data(column, Qt::DisplayRole);
+
+                     QString sa, sb;
+
+                     if(ctx)
+                     {
+                       sa = RichResourceTextFormat(*ctx, va);
+                       sb = RichResourceTextFormat(*ctx, vb);
+                     }
+                     else
+                     {
+                       sa = va.toString();
+                       sb = vb.toString();
+                     }
+
+                     bool da_ok = false, db_ok = false;
+                     double da = sa.toDouble(&da_ok);
+                     double db = sb.toDouble(&db_ok);
+
+                     int comp;
+
+                     if(da_ok && db_ok)
+                     {
+                       if(order == Qt::AscendingOrder)
+                         return da < db;
+                       return da > db;
+                     }
+                     else
+                     {
+                       comp = QString::compare(sa, sb, Qt::CaseInsensitive);
+                     }
+
+                     if(order == Qt::AscendingOrder)
+                       return comp < 0;
+                     return comp > 0;
+                   });
 
   for(RDTreeWidgetItem *child : m_children)
     child->sort(column, order);
@@ -356,7 +393,7 @@ QVariant RDTreeWidgetItem::data(int column, int role) const
 
     return QVariant();
   }
-  else if(role == Qt::ToolTipRole && !m_widget->m_instantTooltips)
+  else if(role == Qt::ToolTipRole)
   {
     if(!m_tooltip.isEmpty())
       return m_tooltip;
@@ -570,12 +607,6 @@ RDTreeWidgetItemIterator &RDTreeWidgetItemIterator::operator++()
 
 RDTreeWidget::RDTreeWidget(QWidget *parent) : RDTreeView(parent)
 {
-  // we'll call this ourselves in drawBranches()
-  RDTreeView::enableBranchRectFill(false);
-
-  m_delegate = new RichTextViewDelegate(this);
-  RDTreeView::setItemDelegate(m_delegate);
-
   header()->setSectionsMovable(false);
 
   m_root = new RDTreeWidgetItem;
@@ -597,7 +628,9 @@ RDTreeWidget::RDTreeWidget(QWidget *parent) : RDTreeView(parent)
 
 RDTreeWidget::~RDTreeWidget()
 {
+  blockSignals(true);
   RDTreeView::setModel(NULL);
+  blockSignals(false);
 
   delete m_root;
   delete m_model;
@@ -658,16 +691,18 @@ void RDTreeWidget::setColumnAlignment(int column, Qt::Alignment align)
   m_alignments[column] = align;
 }
 
-void RDTreeWidget::setItemDelegate(QAbstractItemDelegate *delegate)
+RDTreeWidgetItem *RDTreeWidget::itemForIndex(QModelIndex idx) const
 {
-  m_userDelegate = delegate;
-  m_delegate->setForwardDelegate(m_userDelegate);
+  if(idx.model() == m_model)
+    return m_model->itemForIndex(idx);
+  return NULL;
 }
 
-QAbstractItemDelegate *RDTreeWidget::itemDelegate() const
+void RDTreeWidget::copyItem(QPoint pos, RDTreeWidgetItem *item)
 {
-  return m_userDelegate;
+  copyIndex(pos, m_model->indexForItem(item, 0));
 }
+
 void RDTreeWidget::setColumns(const QStringList &columns)
 {
   m_headers = columns;
@@ -750,6 +785,7 @@ void RDTreeWidget::clear()
 void RDTreeWidget::mouseMoveEvent(QMouseEvent *e)
 {
   RDTreeWidgetItem *oldHover = m_model->itemForIndex(m_currentHoverIndex);
+  QModelIndex oldHoverIndex = m_currentHoverIndex;
 
   RDTreeView::mouseMoveEvent(e);
 
@@ -759,12 +795,8 @@ void RDTreeWidget::mouseMoveEvent(QMouseEvent *e)
   {
     setCursor(QCursor(Qt::PointingHandCursor));
   }
-  else if(m_delegate->linkHover(e, font(), m_currentHoverIndex))
-  {
-    m_model->itemChanged(m_model->itemForIndex(m_currentHoverIndex), {Qt::DecorationRole});
-    setCursor(QCursor(Qt::PointingHandCursor));
-  }
-  else
+  else if(oldHoverIndex.column() == m_hoverColumn &&
+          m_currentHoverIndex.column() != m_hoverColumn && m_hoverHandCursor)
   {
     unsetCursor();
   }
@@ -778,27 +810,6 @@ void RDTreeWidget::mouseMoveEvent(QMouseEvent *e)
     m_model->itemChanged(oldHover, roles);
   m_model->itemChanged(newHover, roles);
 
-  if(m_instantTooltips)
-  {
-    QToolTip::hideText();
-
-    if(newHover && !newHover->m_tooltip.isEmpty())
-    {
-      // the documentation says:
-      //
-      // "If text is empty the tool tip is hidden. If the text is the same as the currently shown
-      // tooltip, the tip will not move. You can force moving by first hiding the tip with an
-      // empty
-      // text, and then showing the new tip at the new position."
-      //
-      // However the actual implementation has some kind of 'fading' check, so if you hide then
-      // immediately show, it will try to reuse the tooltip and end up not moving it at all if the
-      // text hasn't changed.
-      QToolTip::showText(QCursor::pos(), lit(" "), this);
-      QToolTip::showText(QCursor::pos(), newHover->m_tooltip, this);
-    }
-  }
-
   emit hoverItemChanged(newHover);
 
   emit mouseMove(e);
@@ -825,8 +836,6 @@ void RDTreeWidget::leaveEvent(QEvent *e)
   if(m_currentHoverIndex.isValid())
   {
     RDTreeWidgetItem *item = m_model->itemForIndex(m_currentHoverIndex);
-    if(!item->m_tooltip.isEmpty() && m_instantTooltips)
-      QToolTip::hideText();
     m_model->itemChanged(item, {Qt::DecorationRole, Qt::BackgroundRole, Qt::ForegroundRole});
   }
 
@@ -838,79 +847,14 @@ void RDTreeWidget::leaveEvent(QEvent *e)
 void RDTreeWidget::focusOutEvent(QFocusEvent *event)
 {
   if(m_clearSelectionOnFocusLoss)
+  {
+    setCurrentItem(NULL);
     clearSelection();
+  }
 
   RDTreeView::focusOutEvent(event);
 }
 
-void RDTreeWidget::drawBranches(QPainter *painter, const QRect &rect, const QModelIndex &index) const
-{
-  // we do our own custom branch rendering to ensure the backgrounds for the +/- markers are
-  // filled
-  // (as otherwise they don't show up well over selection or background fills) as well as to draw
-  // any vertical branch colors.
-
-  // start at the left-most side of the rect
-  QRect branchRect(rect.left(), rect.top(), indentation(), rect.height());
-
-  RDTreeWidgetItem *item = m_model->itemForIndex(index);
-
-  // first draw the coloured lines - we're only interested in parents for this, so push all the
-  // parents onto a stack
-  QStack<RDTreeWidgetItem *> parents;
-
-  RDTreeWidgetItem *parent = item->parent();
-
-  while(parent && parent != m_root)
-  {
-    parents.push(parent);
-    parent = parent->parent();
-  }
-
-  // fill in the background behind the lines for the whole row, since by default it doesn't show
-  // up
-  // behind the tree lines.
-
-  QRect allLinesRect(rect.left(), rect.top(), (parents.count() + 1) * indentation(), rect.height());
-
-  // calling this manually here means it won't be called later in RDTreeView::drawBranches, and
-  // allows us to overwrite it with our background filling if we want to.
-  RDTreeWidget::fillBranchesRect(painter, rect, index);
-
-  if(!selectionModel()->isSelected(index) && item->m_back != QBrush())
-  {
-    painter->fillRect(allLinesRect, item->m_back);
-  }
-
-  RDTreeView::drawBranches(painter, rect, index);
-
-  // we now iterate from the top-most parent down, moving in from the left
-  // we draw this after calling into drawBranches() so we paint on top of the built-in lines
-  QPen oldPen = painter->pen();
-  while(!parents.isEmpty())
-  {
-    parent = parents.pop();
-
-    if(parent->m_treeCol.isValid())
-    {
-      // draw a centred pen vertically down the middle of branchRect
-
-      painter->setPen(QPen(QBrush(parent->m_treeCol), parent->m_treeColWidth));
-
-      QPoint topCentre = QRect(branchRect).center();
-      QPoint bottomCentre = topCentre;
-
-      topCentre.setY(branchRect.top());
-      bottomCentre.setY(branchRect.bottom());
-
-      painter->drawLine(topCentre, bottomCentre);
-    }
-
-    branchRect.moveLeft(branchRect.left() + indentation());
-  }
-  painter->setPen(oldPen);
-}
-
 void RDTreeWidget::selectionChanged(const QItemSelection &selected, const QItemSelection &deselected)
 {
   emit itemSelectionChanged();
