@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -32,18 +32,17 @@ ResourcePreview::ResourcePreview(ICaptureContext &c, IReplayOutput *output, QWid
 {
   ui->setupUi(this);
 
-  CustomPaintWidget *thumb = new CustomPaintWidget(&c, this);
-  thumb->setOutput(output);
-  thumb->setObjectName(ui->thumbnail->objectName());
-  thumb->setSizePolicy(ui->thumbnail->sizePolicy());
-  thumb->setMinimumSize(QSize(0, 0));
-
-  delete ui->thumbnail;
-  ui->thumbnail = thumb;
-  ui->gridLayout->addWidget(ui->thumbnail, 0, 0, 1, 2);
+  ui->thumbnail->SetContext(c);
+  ui->thumbnail->SetOutput(output);
 
   setBackgroundRole(QPalette::Background);
-  setForegroundRole(QPalette::Highlight);
+  setForegroundRole(QPalette::Foreground);
+
+  QPalette Pal(palette());
+
+  Pal.setColor(QPalette::Highlight, QColor(Qt::red));
+
+  setPalette(Pal);
 
   setSelected(false);
 
@@ -106,20 +105,10 @@ void ResourcePreview::setSelected(bool sel)
 {
   m_Selected = sel;
 
-  QPalette Pal(palette());
-
-  Pal.setColor(QPalette::Highlight, sel ? QColor(Qt::red) : Pal.color(QPalette::Foreground));
-
-  setPalette(Pal);
+  setForegroundRole(sel ? QPalette::Highlight : QPalette::Foreground);
 }
 
-void ResourcePreview::changeEvent(QEvent *event)
+WindowingData ResourcePreview::GetWidgetWindowingData()
 {
-  if(event->type() == QEvent::PaletteChange || event->type() == QEvent::StyleChange)
-    setSelected(m_Selected);
-}
-
-QWidget *ResourcePreview::thumbWidget()
-{
-  return ui->thumbnail;
+  return ui->thumbnail->GetWidgetWindowingData();
 }
