@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,6 +23,7 @@
  ******************************************************************************/
 
 #include "CaptureDialog.h"
+#include <QKeyEvent>
 #include <QMouseEvent>
 #include <QSortFilterProxyModel>
 #include <QStandardItemModel>
@@ -94,7 +95,7 @@ void CaptureDialog::initWarning(RDLabel *warning)
 
 QString CaptureDialog::mostRecentFilename()
 {
-  return configFilePath(lit("most_recent.cap"));
+  return ConfigFilePath(lit("most_recent.cap"));
 }
 
 void CaptureDialog::PopulateMostRecent()
@@ -207,6 +208,10 @@ CaptureDialog::CaptureDialog(ICaptureContext &ctx, OnCaptureMethod captureCallba
   initWarning(ui->androidWarn);
   QObject::connect(ui->androidWarn, &RDLabel::clicked, this, &CaptureDialog::androidWarn_mouseClick);
 
+  QObject::connect(ui->exePath, &RDLineEdit::keyPress, this, &CaptureDialog::lineEdit_keyPress);
+  QObject::connect(ui->workDirPath, &RDLineEdit::keyPress, this, &CaptureDialog::lineEdit_keyPress);
+  QObject::connect(ui->cmdline, &RDLineEdit::keyPress, this, &CaptureDialog::lineEdit_keyPress);
+
   m_AndroidFlags = AndroidFlags::NoFlags;
 
   m_CaptureCallback = captureCallback;
@@ -270,12 +275,12 @@ void CaptureDialog::on_CaptureCallstacks_toggled(bool checked)
 {
   if(ui->CaptureCallstacks->isChecked())
   {
-    ui->CaptureCallstacksOnlyDraws->setEnabled(true);
+    ui->CaptureCallstacksOnlyActions->setEnabled(true);
   }
   else
   {
-    ui->CaptureCallstacksOnlyDraws->setChecked(false);
-    ui->CaptureCallstacksOnlyDraws->setEnabled(false);
+    ui->CaptureCallstacksOnlyActions->setChecked(false);
+    ui->CaptureCallstacksOnlyActions->setEnabled(false);
   }
 }
 
@@ -565,7 +570,7 @@ void CaptureDialog::CheckAndroidSetup(QString &filename)
   LambdaThread *scan = new LambdaThread([this, filename]() {
 
     rdcstr host = m_Ctx.Replay().CurrentRemote().Hostname();
-    RENDERDOC_CheckAndroidPackage(host.c_str(), filename.toUtf8().data(), &m_AndroidFlags);
+    RENDERDOC_CheckAndroidPackage(host, filename, &m_AndroidFlags);
 
     const bool debuggable = bool(m_AndroidFlags & AndroidFlags::Debuggable);
     const bool hasroot = bool(m_AndroidFlags & AndroidFlags::RootAccess);
@@ -631,8 +636,8 @@ Would you like RenderDoc to try patching your package?
 
     // call into APK pull, patch, install routine, then continue
     LambdaThread *patch = new LambdaThread([this, host, exe, &patchSucceeded, &progress]() {
-      AndroidFlags result = RENDERDOC_MakeDebuggablePackage(host.c_str(), exe.toUtf8().data(),
-                                                            [&progress](float p) { progress = p; });
+      AndroidFlags result =
+          RENDERDOC_MakeDebuggablePackage(host, exe, [&progress](float p) { progress = p; });
 
       if(result & AndroidFlags::Debuggable)
       {
@@ -684,6 +689,15 @@ Would you like RenderDoc to try patching your package?
   }
 }
 
+void CaptureDialog::lineEdit_keyPress(QKeyEvent *ev)
+{
+  if((ev->key() == Qt::Key_Return || ev->key() == Qt::Key_Enter) &&
+     ev->modifiers() & Qt::ControlModifier)
+  {
+    TriggerCapture();
+  }
+}
+
 void CaptureDialog::on_processRefesh_clicked()
 {
   fillProcessList();
@@ -879,8 +893,7 @@ void CaptureDialog::on_toggleGlobal_clicked()
 
     QString capturefile = m_Ctx.TempCaptureFilename(QFileInfo(exe).baseName());
 
-    bool success = RENDERDOC_StartGlobalHook(exe.toUtf8().data(), capturefile.toUtf8().data(),
-                                             Settings().options);
+    bool success = RENDERDOC_StartGlobalHook(exe, capturefile, Settings().options);
 
     if(!success)
     {
@@ -973,7 +986,7 @@ void CaptureDialog::SetSettings(CaptureSettings settings)
   ui->AllowVSync->setChecked(settings.options.allowVSync);
   ui->HookIntoChildren->setChecked(settings.options.hookIntoChildren);
   ui->CaptureCallstacks->setChecked(settings.options.captureCallstacks);
-  ui->CaptureCallstacksOnlyDraws->setChecked(settings.options.captureCallstacksOnlyDraws);
+  ui->CaptureCallstacksOnlyActions->setChecked(settings.options.captureCallstacksOnlyActions);
   ui->APIValidation->setChecked(settings.options.apiValidation);
   ui->RefAllResources->setChecked(settings.options.refAllResources);
   ui->CaptureAllCmdLists->setChecked(settings.options.captureAllCmdLists);
@@ -1021,7 +1034,7 @@ CaptureSettings CaptureDialog::Settings()
   ret.options.allowVSync = ui->AllowVSync->isChecked();
   ret.options.hookIntoChildren = ui->HookIntoChildren->isChecked();
   ret.options.captureCallstacks = ui->CaptureCallstacks->isChecked();
-  ret.options.captureCallstacksOnlyDraws = ui->CaptureCallstacksOnlyDraws->isChecked();
+  ret.options.captureCallstacksOnlyActions = ui->CaptureCallstacksOnlyActions->isChecked();
   ret.options.apiValidation = ui->APIValidation->isChecked();
   ret.options.refAllResources = ui->RefAllResources->isChecked();
   ret.options.captureAllCmdLists = ui->CaptureAllCmdLists->isChecked();
