@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -685,12 +685,11 @@ QString LiveCapture::MakeText(Capture *cap)
   return text;
 }
 
-bool LiveCapture::checkAllowClose()
+bool LiveCapture::checkAllowClose(bool multipleClosures, bool &noToAll)
 {
   m_IgnoreThreadClosed = true;
 
   bool suppressRemoteWarning = false;
-  bool notoall = false;
 
   QMessageBox::StandardButtons msgFlags = RDDialog::YesNoCancel;
 
@@ -712,7 +711,7 @@ bool LiveCapture::checkAllowClose()
 
     QMessageBox::StandardButton res = QMessageBox::No;
 
-    if(!suppressRemoteWarning && !notoall)
+    if(!suppressRemoteWarning && !noToAll)
     {
       QString frameName = tr("Frame #%1").arg(cap->frameNumber);
       if(cap->frameNumber == ~0U)
@@ -727,7 +726,26 @@ bool LiveCapture::checkAllowClose()
 
       if(res == QMessageBox::NoToAll)
       {
-        notoall = true;
+        // if we're closing multiple connections make sure the user is sure of what they're doing
+        if(multipleClosures)
+        {
+          QMessageBox::StandardButton res2 =
+              RDDialog::question(this, tr("Discarding all captures"),
+                                 tr("Multiple connections open have potentially unsaved captures, "
+                                    "are you sure you wish to discard them all?"));
+
+          // if the user is sure, apply the no to all
+          if(res2 == QMessageBox::Yes)
+            noToAll = true;
+
+          // otherwise we'll treat this as a simple 'no' in case they changed their mind.
+        }
+        else
+        {
+          // if we're not closing multiple, we can just immediately accept the 'no to all'
+          noToAll = true;
+        }
+
         res = QMessageBox::No;
       }
     }
@@ -782,6 +800,12 @@ bool LiveCapture::checkAllowClose()
   return true;
 }
 
+bool LiveCapture::checkAllowClose()
+{
+  bool dummy = false;
+  return checkAllowClose(false, dummy);
+}
+
 void LiveCapture::openCapture(Capture *cap)
 {
   cap->opened = true;
@@ -801,6 +825,16 @@ void LiveCapture::openCapture(Capture *cap)
 
 bool LiveCapture::saveCapture(Capture *cap, QString path)
 {
+  // if this is the current capture, do the save through the main window
+  if(QString(m_Ctx.GetCaptureFilename()) == cap->path)
+  {
+    // if there's no target path, let the main window prompt for save.
+    if(path.isEmpty())
+      return m_Main->PromptSaveCaptureAs();
+    else
+      return m_Main->SaveCurrentCapture(path);
+  }
+
   if(path.isEmpty())
   {
     path = m_Main->GetSavePath();
@@ -926,6 +960,21 @@ void LiveCapture::cleanItems()
   ui->captures->clear();
 }
 
+void LiveCapture::fileSaved(QString from, QString to)
+{
+  for(int i = 0; i < ui->captures->count(); i++)
+  {
+    Capture *cap = GetCapture(ui->captures->item(i));
+
+    if(cap->path == from)
+    {
+      cap->path = to;
+      cap->saved = true;
+      cap->local = true;
+    }
+  }
+}
+
 void LiveCapture::previewToggle_toggled(bool checked)
 {
   if(m_IgnorePreviewToggle)
@@ -1159,8 +1208,8 @@ void LiveCapture::selfClose()
 
 void LiveCapture::connectionThreadEntry()
 {
-  ITargetControl *conn = RENDERDOC_CreateTargetControl(m_Hostname.toUtf8().data(), m_RemoteIdent,
-                                                       GetSystemUsername().toUtf8().data(), true);
+  ITargetControl *conn =
+      RENDERDOC_CreateTargetControl(m_Hostname, m_RemoteIdent, GetSystemUsername(), true);
   m_Connected.release();
 
   if(!conn || !conn->Connected())
@@ -1181,7 +1230,7 @@ void LiveCapture::connectionThreadEntry()
   }
 
   uint32_t pid = conn->GetPID();
-  QString target = QString::fromUtf8(conn->GetTarget());
+  QString target = conn->GetTarget();
 
   GUIInvoke::call(this, [this, pid, target]() {
     if(!m_Connected.available())
@@ -1214,7 +1263,7 @@ void LiveCapture::connectionThreadEntry()
 
     if(m_CopyCapture.tryAcquire())
     {
-      conn->CopyCapture(m_CopyCaptureID, m_CopyCaptureLocalPath.toUtf8().data());
+      conn->CopyCapture(m_CopyCaptureID, m_CopyCaptureLocalPath);
       m_CopyCaptureLocalPath = QString();
       m_CopyCaptureID = ~0U;
     }
@@ -1305,7 +1354,7 @@ void LiveCapture::connectionThreadEntry()
       NewCaptureData cap = msg.newCapture;
       if(cap.api.isEmpty())
         cap.api = conn->GetAPI();
-      QString name = QString::fromUtf8(conn->GetTarget());
+      QString name = conn->GetTarget();
       GUIInvoke::call(this, [this, name, cap]() { captureAdded(name, cap); });
     }
 
@@ -1341,6 +1390,11 @@ void LiveCapture::connectionThreadEntry()
       uint32_t windows = msg.capturableWindowCount;
       GUIInvoke::call(this, [this, windows]() { ui->cycleActiveWindow->setEnabled(windows > 1); });
     }
+
+    if(msg.type == TargetControlMessageType::RequestShow)
+    {
+      GUIInvoke::call(this, [this]() { m_Main->BringToFront(); });
+    }
   }
 
   if(conn)
