@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -25,6 +25,7 @@
 #include "SettingsDialog.h"
 #include <float.h>
 #include <math.h>
+#include <QFontDatabase>
 #include <QKeyEvent>
 #include <QTextEdit>
 #include <QToolButton>
@@ -59,6 +60,65 @@ SettingsDialog::SettingsDialog(ICaptureContext &ctx, QWidget *parent)
   for(int i = 0; i < StyleData::numAvailable; i++)
     ui->UIStyle->addItem(StyleData::availStyles[i].styleName);
 
+  QFontDatabase fontdb;
+
+  QStringList fontFamilies = fontdb.families();
+  fontFamilies.insert(0, tr("Default (%1)").arg(Formatter::DefaultFontFamily()));
+
+  ui->Font_Family->addItems(fontFamilies);
+
+  int curFontOption = -1;
+  for(int i = 0; i < ui->Font_Family->count(); i++)
+  {
+    if(ui->Font_Family->itemText(i) == m_Ctx.Config().Font_Family)
+    {
+      curFontOption = i;
+      break;
+    }
+  }
+
+  if(m_Ctx.Config().Font_Family.isEmpty() || curFontOption < 0)
+    curFontOption = 0;
+
+  ui->Font_Family->setCurrentIndex(curFontOption);
+
+  // remove the default again
+  fontFamilies.removeAt(0);
+
+  // remove any non-fixed width fonts
+  for(int i = 0; i < fontFamilies.count();)
+  {
+    if(!fontdb.isFixedPitch(fontFamilies[i]))
+    {
+      fontFamilies.removeAt(i);
+      // check i again
+      continue;
+    }
+
+    // move to the next
+    i++;
+  }
+
+  // re-add the default
+  fontFamilies.insert(0, tr("Default (%1)").arg(Formatter::DefaultMonoFontFamily()));
+
+  ui->Font_MonoFamily->addItems(fontFamilies);
+
+  curFontOption = -1;
+  for(int i = 0; i < ui->Font_MonoFamily->count(); i++)
+  {
+    if(ui->Font_MonoFamily->itemText(i) == m_Ctx.Config().Font_MonoFamily)
+    {
+      curFontOption = i;
+      break;
+    }
+  }
+
+  if(m_Ctx.Config().Font_MonoFamily.isEmpty() || curFontOption < 0)
+    curFontOption = 0;
+
+  ui->Font_MonoFamily->setCurrentIndex(curFontOption);
+
   ui->Font_GlobalScale->addItems({lit("50%"), lit("75%"), lit("100%"), lit("125%"), lit("150%"),
                                   lit("175%"), lit("200%"), lit("250%"), lit("300%"), lit("400%")});
 
@@ -221,17 +281,13 @@ SettingsDialog::SettingsDialog(ICaptureContext &ctx, QWidget *parent)
 #endif
 
   ui->AllowGlobalHook->setChecked(m_Ctx.Config().AllowGlobalHook);
+  ui->AllowProcessInject->setChecked(m_Ctx.Config().AllowProcessInject);
 
   ui->EventBrowser_TimeUnit->setCurrentIndex((int)m_Ctx.Config().EventBrowser_TimeUnit);
   ui->EventBrowser_AddFake->setChecked(m_Ctx.Config().EventBrowser_AddFake);
-  ui->EventBrowser_HideEmpty->setChecked(m_Ctx.Config().EventBrowser_HideEmpty);
-  ui->EventBrowser_HideAPICalls->setChecked(m_Ctx.Config().EventBrowser_HideAPICalls);
   ui->EventBrowser_ApplyColors->setChecked(m_Ctx.Config().EventBrowser_ApplyColors);
   ui->EventBrowser_ColorEventRow->setChecked(m_Ctx.Config().EventBrowser_ColorEventRow);
 
-  // disable sub-checkbox
-  ui->EventBrowser_ColorEventRow->setEnabled(ui->EventBrowser_ApplyColors->isChecked());
-
   ui->Comments_ShowOnLoad->setChecked(m_Ctx.Config().Comments_ShowOnLoad);
 
   ui->Formatter_MinFigures->setValue(m_Ctx.Config().Formatter_MinFigures);
@@ -248,6 +304,12 @@ SettingsDialog::SettingsDialog(ICaptureContext &ctx, QWidget *parent)
     ui->globalHookLabel->setToolTip(disabledTooltip);
   }
 
+// process injection is not supported on non-Windows
+#if !defined(Q_OS_WIN32)
+  ui->injectProcLabel->setVisible(false);
+  ui->AllowProcessInject->setVisible(false);
+#endif
+
   m_Init = false;
 
   QObject::connect(ui->Font_GlobalScale->lineEdit(), &QLineEdit::returnPressed, this,
@@ -309,6 +371,36 @@ void SettingsDialog::on_okButton_accepted()
   accept();
 }
 
+void SettingsDialog::on_Font_Family_currentIndexChanged(int index)
+{
+  if(m_Init)
+    return;
+
+  if(index == 0)
+    m_Ctx.Config().Font_Family.clear();
+  else
+    m_Ctx.Config().Font_Family = ui->Font_Family->currentText();
+
+  m_Ctx.Config().SetupFormatting();
+
+  m_Ctx.Config().Save();
+}
+
+void SettingsDialog::on_Font_MonoFamily_currentIndexChanged(int index)
+{
+  if(m_Init)
+    return;
+
+  if(index == 0)
+    m_Ctx.Config().Font_MonoFamily.clear();
+  else
+    m_Ctx.Config().Font_MonoFamily = ui->Font_MonoFamily->currentText();
+
+  m_Ctx.Config().SetupFormatting();
+
+  m_Ctx.Config().Save();
+}
+
 void SettingsDialog::on_Font_GlobalScale_currentIndexChanged(int index)
 {
   Font_GlobalScale_returnPressed();
@@ -390,6 +482,16 @@ void SettingsDialog::on_AllowGlobalHook_toggled(bool checked)
     m_Ctx.GetCaptureDialog()->UpdateGlobalHook();
 }
 
+void SettingsDialog::on_AllowProcessInject_toggled(bool checked)
+{
+  m_Ctx.Config().AllowProcessInject = ui->AllowProcessInject->isChecked();
+
+  m_Ctx.Config().Save();
+
+  if(m_Ctx.HasCaptureDialog())
+    m_Ctx.GetCaptureDialog()->UpdateGlobalHook();
+}
+
 void SettingsDialog::on_CheckUpdate_AllowChecks_toggled(bool checked)
 {
   m_Ctx.Config().CheckUpdate_AllowChecks = ui->CheckUpdate_AllowChecks->isChecked();
@@ -803,6 +905,8 @@ bool SettingsDialog::editTool(int existing, ShaderProcessingTool &tool)
 
     QString message;
 
+    invalid = false;
+
     // ensure we don't have an invalid name
     if(tool.name == "Builtin")
     {
@@ -983,6 +1087,9 @@ void SettingsDialog::on_EventBrowser_TimeUnit_currentIndexChanged(int index)
   if(m_Ctx.HasEventBrowser())
     m_Ctx.GetEventBrowser()->UpdateDurationColumn();
 
+  if(m_Ctx.HasPerformanceCounterViewer())
+    m_Ctx.GetPerformanceCounterViewer()->UpdateDurationColumn();
+
   m_Ctx.Config().Save();
 }
 
@@ -993,24 +1100,13 @@ void SettingsDialog::on_EventBrowser_AddFake_toggled(bool checked)
   m_Ctx.Config().Save();
 }
 
-void SettingsDialog::on_EventBrowser_HideEmpty_toggled(bool checked)
-{
-  m_Ctx.Config().EventBrowser_HideEmpty = ui->EventBrowser_HideEmpty->isChecked();
-
-  m_Ctx.Config().Save();
-}
-
-void SettingsDialog::on_EventBrowser_HideAPICalls_toggled(bool checked)
-{
-  m_Ctx.Config().EventBrowser_HideAPICalls = ui->EventBrowser_HideAPICalls->isChecked();
-
-  m_Ctx.Config().Save();
-}
-
 void SettingsDialog::on_EventBrowser_ApplyColors_toggled(bool checked)
 {
   m_Ctx.Config().EventBrowser_ApplyColors = ui->EventBrowser_ApplyColors->isChecked();
 
+  // disable sub-checkbox
+  ui->EventBrowser_ColorEventRow->setEnabled(ui->EventBrowser_ApplyColors->isChecked());
+
   m_Ctx.Config().Save();
 }
 
