@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -82,6 +82,8 @@ public:
   QWidget *Widget() override { return this; }
   void RegisterShortcut(const rdcstr &shortcut, QWidget *widget, ShortcutCallback callback) override;
   void UnregisterShortcut(const rdcstr &shortcut, QWidget *widget) override;
+  void BringToFront() override;
+
   // ICaptureViewer
   void OnCaptureLoaded() override;
   void OnCaptureClosed() override;
@@ -94,6 +96,8 @@ public:
 
   void LoadInitialLayout();
 
+  void sendErrorReport(bool forceCaptureInclusion);
+
   void setProgress(float val);
   void setRemoteHost(int hostIdx);
   void takeCaptureOwnership() { m_OwnTempCapture = true; }
@@ -113,6 +117,10 @@ public:
   void ShowLiveCapture(LiveCapture *live);
   void LiveCaptureClosed(LiveCapture *live);
 
+  bool PromptCloseCapture();
+  bool PromptSaveCaptureAs();
+  bool SaveCurrentCapture(QString saveFilename);
+
   void RemoveRecentCapture(const QString &filename);
 
   QMenu *GetBaseMenu(WindowMenu base, rdcstr name);
@@ -277,8 +285,6 @@ private:
   void recentCaptureFile(const QString &filename);
   void recentCaptureSetting(const QString &filename);
 
-  bool PromptCloseCapture();
-  bool PromptSaveCaptureAs();
   void OpenCaptureConfigFile(const QString &filename, bool exe);
 
   QVariantMap saveState();
