@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -34,6 +34,7 @@ class D3D12PipelineStateViewer;
 
 class QXmlStreamWriter;
 
+class ComputeDebugSelector;
 class RDLabel;
 class RDTreeWidget;
 class RDTreeWidgetItem;
@@ -56,6 +57,7 @@ public:
   void OnEventChanged(uint32_t eventId) override;
 
   void SelectPipelineStage(PipelineStage stage);
+  ResourceId GetResource(RDTreeWidgetItem *item);
 
 private slots:
   // automatic slots
@@ -76,20 +78,23 @@ private slots:
   void cbuffer_itemActivated(RDTreeWidgetItem *item, int column);
   void vertex_leave(QEvent *e);
 
-  void on_debugThread_clicked();
+  void on_computeDebugSelector_clicked();
+  void computeDebugSelector_beginDebug(const rdcfixedarray<uint32_t, 3> &group,
+                                       const rdcfixedarray<uint32_t, 3> &thread);
 
 private:
   Ui::D3D12PipelineStateViewer *ui;
   ICaptureContext &m_Ctx;
   PipelineStateViewer &m_Common;
+  ComputeDebugSelector *m_ComputeDebugSelector;
 
   void setShaderState(const rdcarray<D3D12Pipe::RootSignatureRange> &rootElements,
                       const D3D12Pipe::Shader &stage, RDLabel *shader, RDLabel *rootSig,
                       RDTreeWidget *tex, RDTreeWidget *samp, RDTreeWidget *cbuffer,
                       RDTreeWidget *uavs);
 
-  void addResourceRow(const D3D12ViewTag &view, const D3D12Pipe::Shader *stage,
-                      RDTreeWidget *resources);
+  void addResourceRow(const D3D12ViewTag &view, const Bindpoint *bind,
+                      const ShaderResource *shaderInput, RDTreeWidget *resources);
 
   void clearShaderState(RDLabel *shader, RDLabel *rootSig, RDTreeWidget *tex, RDTreeWidget *samp,
                         RDTreeWidget *cbuffer, RDTreeWidget *uavs);
@@ -104,9 +109,13 @@ private:
 
   void setViewDetails(RDTreeWidgetItem *node, const D3D12ViewTag &view, TextureDescription *tex);
   void setViewDetails(RDTreeWidgetItem *node, const D3D12ViewTag &view, BufferDescription *buf);
+  bool isByteAddress(const D3D12Pipe::View &r, const ShaderResource *shaderInput);
 
   bool showNode(bool usedSlot, bool filledSlot);
 
+  bool m_ShowUnused = false;
+  bool m_ShowEmpty = false;
+
   QVariantList exportViewHTML(const D3D12Pipe::View &view, bool rw,
                               const ShaderResource *shaderInput, const QString &extraParams);
   void exportHTML(QXmlStreamWriter &xml, const D3D12Pipe::InputAssembly &ia);
