@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -26,6 +26,7 @@
 #include <QMenu>
 #include <QMouseEvent>
 #include <QPainter>
+#include <QStylePainter>
 #include <QSvgRenderer>
 #include <QToolButton>
 #include <QXmlStreamWriter>
@@ -80,11 +81,122 @@ static uint32_t byteSize(const ResourceFormat &fmt)
   return fmt.compByteWidth * fmt.compCount;
 }
 
+RDPreviewTooltip::RDPreviewTooltip(PipelineStateViewer *parent, CustomPaintWidget *thumbnail,
+                                   ICaptureContext &ctx)
+    : QFrame(parent), m_Ctx(ctx)
+{
+  int margin = style()->pixelMetric(QStyle::PM_ToolTipLabelFrameWidth, NULL, this);
+  int opacity = style()->styleHint(QStyle::SH_ToolTipLabel_Opacity, NULL, this);
+
+  pipe = parent;
+
+  setWindowFlags(Qt::ToolTip);
+  setAttribute(Qt::WA_TransparentForMouseEvents);
+  setForegroundRole(QPalette::ToolTipText);
+  setBackgroundRole(QPalette::ToolTipBase);
+  setFrameStyle(QFrame::NoFrame);
+  setWindowOpacity(opacity / 255.0);
+  setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
+
+  QHBoxLayout *hbox = new QHBoxLayout;
+  QVBoxLayout *vbox = new QVBoxLayout;
+  hbox->setSpacing(0);
+  hbox->setContentsMargins(0, 0, 0, 0);
+  vbox->setSpacing(2);
+  vbox->setContentsMargins(6, 3, 6, 3);
+
+  label = new QLabel(this);
+  label->setAlignment(Qt::AlignLeft);
+
+  title = new QLabel(this);
+  title->setAlignment(Qt::AlignLeft);
+
+  setLayout(vbox);
+  vbox->addWidget(title);
+  vbox->addLayout(hbox);
+
+  hbox->addWidget(thumbnail);
+  hbox->addStretch();
+
+  vbox->addWidget(label);
+}
+
+void RDPreviewTooltip::hideTip()
+{
+  hide();
+}
+
+QSize RDPreviewTooltip::configureTip(QWidget *widget, QModelIndex idx, QString text)
+{
+  ResourceId id = pipe->updateThumbnail(widget, idx);
+  if(id != ResourceId())
+  {
+    title->setText(m_Ctx.GetResourceName(id));
+    title->show();
+  }
+  else
+  {
+    title->hide();
+  }
+  label->setText(text);
+  label->setVisible(!text.isEmpty());
+  layout()->update();
+  layout()->activate();
+  return minimumSizeHint();
+}
+
+void RDPreviewTooltip::showTip(QPoint pos)
+{
+  move(pos);
+  resize(minimumSize());
+  show();
+}
+
+bool RDPreviewTooltip::forceTip(QWidget *widget, QModelIndex idx)
+{
+  return pipe->hasThumbnail(widget, idx);
+}
+
+void RDPreviewTooltip::paintEvent(QPaintEvent *ev)
+{
+  QStylePainter p(this);
+  QStyleOptionFrame opt;
+  opt.init(this);
+  p.drawPrimitive(QStyle::PE_PanelTipLabel, opt);
+  p.end();
+
+  QWidget::paintEvent(ev);
+}
+
+void RDPreviewTooltip::resizeEvent(QResizeEvent *e)
+{
+  QStyleHintReturnMask frameMask;
+  QStyleOption option;
+  option.init(this);
+  if(style()->styleHint(QStyle::SH_ToolTip_Mask, &option, this, &frameMask))
+    setMask(frameMask.region);
+
+  QWidget::resizeEvent(e);
+}
+
 PipelineStateViewer::PipelineStateViewer(ICaptureContext &ctx, QWidget *parent)
     : QFrame(parent), ui(new Ui::PipelineStateViewer), m_Ctx(ctx)
 {
   ui->setupUi(this);
 
+  ui->thumbnail->SetContext(m_Ctx);
+  ui->layout->removeWidget(ui->thumbnail);
+
+  m_Tooltip = new RDPreviewTooltip(this, ui->thumbnail, m_Ctx);
+
+  QColor c = palette().color(QPalette::ToolTipBase).toRgb();
+
+  m_TexDisplay.backgroundColor = FloatVector();
+  m_TexDisplay.backgroundColor.w = 1.0f;
+
+  // auto-fit and center scale
+  m_TexDisplay.scale = -1.0f;
+
   m_D3D11 = NULL;
   m_D3D12 = NULL;
   m_GL = NULL;
@@ -105,6 +217,8 @@ PipelineStateViewer::~PipelineStateViewer()
   m_Ctx.BuiltinWindowClosed(this);
   m_Ctx.RemoveCaptureViewer(this);
 
+  delete m_Tooltip;
+
   delete ui;
 }
 
@@ -121,6 +235,33 @@ void PipelineStateViewer::OnCaptureLoaded()
 
   if(m_Current)
     m_Current->OnCaptureLoaded();
+
+  if(!m_Ctx.APIProps().remoteReplay)
+  {
+    WindowingData thumbData = ui->thumbnail->GetWidgetWindowingData();
+
+    m_Ctx.Replay().BlockInvoke([thumbData, this](IReplayController *r) {
+      m_Output = r->CreateOutput(thumbData, ReplayOutputType::Texture);
+
+      ui->thumbnail->SetOutput(m_Output);
+
+      RT_UpdateAndDisplay(r);
+    });
+  }
+  else
+  {
+    m_Output = NULL;
+  }
+}
+
+void PipelineStateViewer::RT_UpdateAndDisplay(IReplayController *r)
+{
+  if(m_Output != NULL)
+  {
+    m_Output->SetTextureDisplay(m_TexDisplay);
+
+    GUIInvoke::call(this, [this]() { ui->thumbnail->update(); });
+  }
 }
 
 void PipelineStateViewer::OnCaptureClosed()
@@ -348,23 +489,24 @@ div.stage table tr td { border-right: 1px solid #AAAAAA; background-color: #EEEE
 
             QString context = frameNumber == ~0U ? tr("Capture") : tr("Frame %1").arg(frameNumber);
 
-            const DrawcallDescription *draw = m_Ctx.CurDrawcall();
+            const ActionDescription *action = m_Ctx.CurAction();
 
-            QList<const DrawcallDescription *> drawstack;
-            const DrawcallDescription *parent = draw ? draw->parent : NULL;
+            QList<const ActionDescription *> actionstack;
+            const ActionDescription *parent = action ? action->parent : NULL;
             while(parent)
             {
-              drawstack.push_front(parent);
+              actionstack.push_front(parent);
               parent = parent->parent;
             }
 
-            for(const DrawcallDescription *d : drawstack)
+            for(const ActionDescription *d : actionstack)
             {
-              context += QFormatStr(" > %1").arg(d->name);
+              context += QFormatStr(" > %1").arg(d->customName);
             }
 
-            if(draw)
-              context += QFormatStr(" => %1").arg(draw->name);
+            if(action)
+              context +=
+                  QFormatStr(" => %1").arg(m_Ctx.GetEventBrowser()->GetEventName(action->eventId));
             else
               context += tr(" => Capture Start");
 
@@ -999,6 +1141,182 @@ void PipelineStateViewer::SetupShaderEditButton(QToolButton *button, ResourceId
   button->setMenu(menu);
 }
 
+void PipelineStateViewer::AddResourceUsageEntry(QMenu &menu, uint32_t start, uint32_t end,
+                                                ResourceUsage usage)
+{
+  QAction *item = NULL;
+
+  if(start == end)
+    item = new QAction(
+        QFormatStr("EID %1: %2").arg(start).arg(ToQStr(usage, m_Ctx.APIProps().pipelineType)), this);
+  else
+    item = new QAction(
+        QFormatStr("EID %1-%2: %3").arg(start).arg(end).arg(ToQStr(usage, m_Ctx.APIProps().pipelineType)),
+        this);
+
+  QObject::connect(item, &QAction::triggered, [this, end]() { m_Ctx.SetEventID({}, end, end); });
+
+  menu.addAction(item);
+}
+
+void PipelineStateViewer::ShowResourceContextMenu(RDTreeWidget *widget, const QPoint &pos,
+                                                  ResourceId id, const rdcarray<EventUsage> &usage)
+{
+  RDTreeWidgetItem *item = widget->itemAt(pos);
+
+  QMenu contextMenu(this);
+
+  QAction copy(tr("&Copy"), this);
+
+  contextMenu.addAction(&copy);
+
+  copy.setIcon(Icons::copy());
+
+  QObject::connect(&copy, &QAction::triggered,
+                   [widget, pos, item]() { widget->copyItem(pos, item); });
+
+  QAction usageTitle(tr("Used:"), this);
+  QAction openResourceInspector(tr("Open in Resource Inspector"), this);
+
+  openResourceInspector.setIcon(Icons::link());
+
+  if(id != ResourceId())
+  {
+    contextMenu.addSeparator();
+    contextMenu.addAction(&openResourceInspector);
+    contextMenu.addAction(&usageTitle);
+
+    QObject::connect(&openResourceInspector, &QAction::triggered, [this, id]() {
+      m_Ctx.ShowResourceInspector();
+
+      m_Ctx.GetResourceInspector()->Inspect(id);
+    });
+
+    CombineUsageEvents(m_Ctx, usage,
+                       [this, &contextMenu](uint32_t start, uint32_t end, ResourceUsage use) {
+                         AddResourceUsageEntry(contextMenu, start, end, use);
+                       });
+  }
+
+  RDDialog::show(&contextMenu, widget->viewport()->mapToGlobal(pos));
+}
+
+ResourceId PipelineStateViewer::updateThumbnail(QWidget *widget, QModelIndex idx)
+{
+  ResourceId id;
+
+  if(!m_Output)
+    return id;
+
+  RDTreeWidget *treeWidget = qobject_cast<RDTreeWidget *>(widget);
+  if(treeWidget)
+  {
+    RDTreeWidgetItem *item = treeWidget->itemForIndex(idx);
+
+    if(item)
+    {
+      if(m_D3D11)
+        id = m_D3D11->GetResource(item);
+      else if(m_D3D12)
+        id = m_D3D12->GetResource(item);
+      else if(m_GL)
+        id = m_GL->GetResource(item);
+      else if(m_Vulkan)
+        id = m_Vulkan->GetResource(item);
+    }
+
+    TextureDescription *tex = m_Ctx.GetTexture(id);
+
+    if(tex)
+    {
+      m_TexDisplay.resourceId = id;
+      INVOKE_MEMFN(RT_UpdateAndDisplay);
+
+      float aspect = (float)tex->width / (float)qMax(1U, tex->height);
+
+      // keep height fixed at 100, and make width match the aspect ratio of the texture - up to 21:9
+      // ratio
+      ui->thumbnail->setFixedSize((int)qBound(100.0f, aspect * 100.0f, (21.0f / 9.0f) * 100.0f), 100);
+      ui->thumbnail->show();
+    }
+    else
+    {
+      ui->thumbnail->hide();
+    }
+  }
+
+  return id;
+}
+
+bool PipelineStateViewer::hasThumbnail(QWidget *widget, QModelIndex idx)
+{
+  if(!m_Output)
+    return false;
+
+  RDTreeWidget *treeWidget = qobject_cast<RDTreeWidget *>(widget);
+  if(treeWidget)
+  {
+    ResourceId id;
+
+    RDTreeWidgetItem *item = treeWidget->itemForIndex(idx);
+
+    if(item)
+    {
+      if(m_D3D11)
+        id = m_D3D11->GetResource(item);
+      else if(m_D3D12)
+        id = m_D3D12->GetResource(item);
+      else if(m_GL)
+        id = m_GL->GetResource(item);
+      else if(m_Vulkan)
+        id = m_Vulkan->GetResource(item);
+    }
+
+    if(id != ResourceId() && m_Ctx.GetTexture(id))
+      return true;
+  }
+
+  return false;
+}
+
+void PipelineStateViewer::SetupResourceView(RDTreeWidget *widget)
+{
+  auto handler = [this, widget](const QPoint &pos) {
+    RDTreeWidgetItem *item = widget->itemAt(pos);
+
+    ResourceId id;
+
+    if(m_D3D11)
+      id = m_D3D11->GetResource(item);
+    else if(m_D3D12)
+      id = m_D3D12->GetResource(item);
+    else if(m_GL)
+      id = m_GL->GetResource(item);
+    else if(m_Vulkan)
+      id = m_Vulkan->GetResource(item);
+
+    if(id != ResourceId())
+    {
+      m_Ctx.Replay().AsyncInvoke([this, widget, pos, id](IReplayController *r) {
+        rdcarray<EventUsage> usage = r->GetUsage(id);
+
+        GUIInvoke::call(this, [this, widget, pos, id, usage]() {
+          ShowResourceContextMenu(widget, pos, id, usage);
+        });
+      });
+    }
+    else
+    {
+      ShowResourceContextMenu(widget, pos, id, {});
+    }
+  };
+
+  widget->setContextMenuPolicy(Qt::CustomContextMenu);
+  QObject::connect(widget, &RDTreeWidget::customContextMenuRequested, handler);
+
+  widget->setCustomTooltip(m_Tooltip);
+}
+
 QString PipelineStateViewer::GetVBufferFormatString(uint32_t slot)
 {
   rdcarray<BoundVBuffer> vbs = m_Ctx.CurPipelineState().GetVBuffers();
@@ -1130,6 +1448,7 @@ QString PipelineStateViewer::GetVBufferFormatString(uint32_t slot)
     QString sanitised_name = real_name;
 
     sanitised_name.replace(QLatin1Char('.'), QLatin1Char('_'))
+        .replace(QLatin1Char(':'), QLatin1Char('_'))
         .replace(QLatin1Char('['), QLatin1Char('_'))
         .replace(QLatin1Char(']'), QLatin1Char('_'));
 
