@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,6 +27,7 @@
 #include <QFrame>
 #include <QLabel>
 #include "Code/Interface/QRDInterface.h"
+#include "Widgets/Extended/RDTreeView.h"
 
 namespace Ui
 {
@@ -39,12 +40,40 @@ class QToolButton;
 class QMenu;
 class RDLabel;
 class RDTreeWidgetItem;
+class RDTreeWidget;
+class CustomPaintWidget;
 
 class D3D11PipelineStateViewer;
 class D3D12PipelineStateViewer;
 class GLPipelineStateViewer;
 class VulkanPipelineStateViewer;
 
+class PipelineStateViewer;
+
+class RDPreviewTooltip : public QFrame, public ITreeViewTipDisplay
+{
+private:
+  Q_OBJECT
+
+  PipelineStateViewer *pipe = NULL;
+  QLabel *title = NULL;
+  QLabel *label = NULL;
+  ICaptureContext &m_Ctx;
+
+public:
+  explicit RDPreviewTooltip(PipelineStateViewer *parent, CustomPaintWidget *thumbnail,
+                            ICaptureContext &ctx);
+
+  void hideTip();
+  QSize configureTip(QWidget *widget, QModelIndex idx, QString text);
+  void showTip(QPoint pos);
+  bool forceTip(QWidget *widget, QModelIndex idx);
+
+protected:
+  void paintEvent(QPaintEvent *);
+  void resizeEvent(QResizeEvent *);
+};
+
 class PipelineStateViewer : public QFrame, public IPipelineStateViewer, public ICaptureViewer
 {
   Q_OBJECT
@@ -76,11 +105,16 @@ public:
                              const ShaderBindpointMapping &bindpointMapping,
                              const ShaderReflection *shaderDetails);
 
+  void SetupResourceView(RDTreeWidget *view);
+
   QString GetVBufferFormatString(uint32_t slot);
 
   void setTopologyDiagram(QLabel *diagram, Topology topo);
   void setMeshViewPixmap(RDLabel *meshView);
 
+  ResourceId updateThumbnail(QWidget *widget, QModelIndex idx);
+  bool hasThumbnail(QWidget *widget, QModelIndex idx);
+
   QXmlStreamWriter *beginHTMLExport();
   void exportHTMLTable(QXmlStreamWriter &xml, const QStringList &cols,
                        const QList<QVariantList> &rows);
@@ -98,6 +132,17 @@ private:
 
   QMenu *editMenus[6] = {};
 
+  RDPreviewTooltip *m_Tooltip = NULL;
+
+  TextureDisplay m_TexDisplay;
+  IReplayOutput *m_Output = NULL;
+
+  void RT_UpdateAndDisplay(IReplayController *r);
+
+  void AddResourceUsageEntry(QMenu &menu, uint32_t start, uint32_t end, ResourceUsage usage);
+  void ShowResourceContextMenu(RDTreeWidget *widget, const QPoint &pos, ResourceId id,
+                               const rdcarray<EventUsage> &usage);
+
   QString GenerateHLSLStub(const ShaderBindpointMapping &bindpointMapping,
                            const ShaderReflection *shaderDetails, const QString &entryFunc);
   IShaderViewer *EditShader(ResourceId id, ShaderStage shaderType, const rdcstr &entry,
