@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,6 +27,7 @@
 #include <math.h>
 #include <QAction>
 #include <QMenu>
+#include "Code/Resources.h"
 #include "toolwindowmanager/ToolWindowManager.h"
 #include "ui_PixelHistoryView.h"
 
@@ -133,9 +134,9 @@ public:
     if(isEvent(parent))
     {
       const QList<PixelModification> &mods = getMods(parent);
-      const DrawcallDescription *draw = m_Ctx.GetDrawcall(mods.front().eventId);
+      const ActionDescription *action = m_Ctx.GetAction(mods.front().eventId);
 
-      if(draw && draw->flags & (DrawFlags::Clear | DrawFlags::PassBoundary))
+      if(action && action->flags & (ActionFlags::Clear | ActionFlags::PassBoundary))
         return 0;
 
       return mods.count();
@@ -193,32 +194,32 @@ public:
           if(isEvent(index))
           {
             const QList<PixelModification> &mods = getMods(index);
-            const DrawcallDescription *drawcall = m_Ctx.GetDrawcall(mods.front().eventId);
-            if(!drawcall)
+            const ActionDescription *action = m_Ctx.GetAction(mods.front().eventId);
+            if(!action)
               return QVariant();
 
             QString ret;
-            QList<const DrawcallDescription *> drawstack;
-            const DrawcallDescription *parent = drawcall->parent;
+            QList<const ActionDescription *> actionstack;
+            const ActionDescription *parent = action->parent;
             while(parent)
             {
-              drawstack.push_back(parent);
+              actionstack.push_back(parent);
               parent = parent->parent;
             }
 
-            if(!drawstack.isEmpty())
+            if(!actionstack.isEmpty())
             {
-              ret += lit("> ") + drawstack.back()->name;
+              ret += lit("> ") + actionstack.back()->customName;
 
-              if(drawstack.count() > 3)
+              if(actionstack.count() > 3)
                 ret += lit(" ...");
 
               ret += lit("\n");
 
-              if(drawstack.count() > 2)
-                ret += lit("> ") + drawstack[1]->name + lit("\n");
-              if(drawstack.count() > 1)
-                ret += lit("> ") + drawstack[0]->name + lit("\n");
+              if(actionstack.count() > 2)
+                ret += lit("> ") + actionstack[1]->customName + lit("\n");
+              if(actionstack.count() > 1)
+                ret += lit("> ") + actionstack[0]->customName + lit("\n");
 
               ret += lit("\n");
             }
@@ -230,10 +231,9 @@ public:
             {
               ret += tr("EID %1\n%2\nBound as UAV or copy - potential modification")
                          .arg(mods.front().eventId)
-                         .arg(drawcall->name);
+                         .arg(m_Ctx.GetEventBrowser()->GetEventName(action->eventId));
 
-              if(memcmp(mods[0].preMod.col.uintValue, mods[0].postMod.col.uintValue,
-                        sizeof(uint32_t) * 4) == 0)
+              if(mods[0].preMod.col.uintValue == mods[0].postMod.col.uintValue)
               {
                 ret += tr("\nNo change in tex value");
                 uavnowrite = true;
@@ -249,7 +249,7 @@ public:
 
               ret += tr("EID %1\n%2%3\n%4 Fragments touching pixel\n")
                          .arg(mods.front().eventId)
-                         .arg(drawcall->name)
+                         .arg(m_Ctx.GetEventBrowser()->GetEventName(action->eventId))
                          .arg(failure)
                          .arg(mods.count());
             }
@@ -356,8 +356,7 @@ public:
             passed |= m.Passed();
 
           if(mods[0].directShaderWrite &&
-             memcmp(mods[0].preMod.col.uintValue, mods[0].postMod.col.uintValue,
-                    sizeof(uint32_t) * 4) == 0)
+             mods[0].preMod.col.uintValue == mods[0].postMod.col.uintValue)
             return QBrush(QColor::fromRgb(235, 235, 235));
 
           return passed ? QBrush(QColor::fromRgb(235, 255, 235))
@@ -796,11 +795,12 @@ void PixelHistoryView::jumpToPrimitive(EventTag tag)
 
   IBufferViewer *viewer = m_Ctx.GetMeshPreview();
 
-  const DrawcallDescription *draw = m_Ctx.CurDrawcall();
+  const ActionDescription *action = m_Ctx.CurAction();
 
-  if(draw)
+  if(action)
   {
-    uint32_t vertIdx = RENDERDOC_VertexOffset(draw->topology, tag.primitive);
+    uint32_t vertIdx =
+        RENDERDOC_VertexOffset(m_Ctx.CurPipelineState().GetPrimitiveTopology(), tag.primitive);
 
     if(vertIdx != ~0U)
       viewer->ScrollToRow(vertIdx);
@@ -839,6 +839,8 @@ void PixelHistoryView::on_events_customContextMenuRequested(const QPoint &pos)
 
   QAction jumpAction(tr("&Go to primitive %1 at Event %2").arg(tag.primitive).arg(tag.eventId), this);
 
+  jumpAction.setIcon(Icons::find());
+
   QString debugText;
 
   if(tag.primitive == ~0U)
@@ -859,6 +861,8 @@ void PixelHistoryView::on_events_customContextMenuRequested(const QPoint &pos)
 
   QAction debugAction(debugText, this);
 
+  debugAction.setIcon(Icons::wrench());
+
   contextMenu.addAction(&debugAction);
 
   if(!m_Ctx.APIProps().shaderDebugging)
