@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,6 +27,7 @@
 #include <QKeyEvent>
 #include <QMenu>
 #include <QScrollBar>
+#include "Code/QRDUtils.h"
 #include "Code/ScintillaSyntax.h"
 #include "Code/pyrenderdoc/PythonContext.h"
 #include "scintilla/include/SciLexer.h"
@@ -85,13 +86,23 @@ struct MiniQtInvoker : ObjectForwarder<IMiniQtHelper>
 {
   MiniQtInvoker(PythonShell *shell, IMiniQtHelper &obj) : ObjectForwarder(shell, obj) {}
   virtual ~MiniQtInvoker() {}
+  void InvokeOntoUIThread(std::function<void()> callback)
+  {
+    // this function is already thread safe since it's invoking, so just call it directly
+    m_Obj.InvokeOntoUIThread(callback);
+  }
+
   ///////////////////////////////////////////////////////////////////////
   // all functions invoke onto the UI thread since they deal with widgets!
   ///////////////////////////////////////////////////////////////////////
 
-  QWidget *CreateToplevelWidget(const rdcstr &windowTitle)
+  QWidget *CreateToplevelWidget(const rdcstr &windowTitle, WidgetCallback closed)
   {
-    return InvokeRetFunction<QWidget *>(&IMiniQtHelper::CreateToplevelWidget, windowTitle);
+    return InvokeRetFunction<QWidget *>(&IMiniQtHelper::CreateToplevelWidget, windowTitle, closed);
+  }
+  void CloseToplevelWidget(QWidget *widget)
+  {
+    InvokeVoidFunction(&IMiniQtHelper::CloseToplevelWidget, widget);
   }
 
   // widget hierarchy
@@ -116,15 +127,15 @@ struct MiniQtInvoker : ObjectForwarder<IMiniQtHelper>
   {
     return InvokeRetFunction<QWidget *>(&IMiniQtHelper::GetParent, widget);
   }
-  int GetNumChildren(QWidget *widget)
+  int32_t GetNumChildren(QWidget *widget)
   {
-    return InvokeRetFunction<int>(&IMiniQtHelper::GetNumChildren, widget);
+    return InvokeRetFunction<int32_t>(&IMiniQtHelper::GetNumChildren, widget);
   }
-  QWidget *GetChild(QWidget *parent, int index)
+  QWidget *GetChild(QWidget *parent, int32_t index)
   {
     return InvokeRetFunction<QWidget *>(&IMiniQtHelper::GetChild, parent, index);
   }
-
+  void DestroyWidget(QWidget *widget) { InvokeVoidFunction(&IMiniQtHelper::DestroyWidget, widget); }
   // dialogs
 
   bool ShowWidgetAsDialog(QWidget *widget)
@@ -150,12 +161,16 @@ struct MiniQtInvoker : ObjectForwarder<IMiniQtHelper>
   {
     return InvokeRetFunction<QWidget *>(&IMiniQtHelper::CreateGridContainer);
   }
-
+  QWidget *CreateSpacer(bool horizontal)
+  {
+    return InvokeRetFunction<QWidget *>(&IMiniQtHelper::CreateSpacer, horizontal);
+  }
   void ClearContainedWidgets(QWidget *parent)
   {
     InvokeVoidFunction(&IMiniQtHelper::ClearContainedWidgets, parent);
   }
-  void AddGridWidget(QWidget *parent, int row, int column, QWidget *child, int rowSpan, int columnSpan)
+  void AddGridWidget(QWidget *parent, int32_t row, int32_t column, QWidget *child, int32_t rowSpan,
+                     int32_t columnSpan)
   {
     InvokeVoidFunction(&IMiniQtHelper::AddGridWidget, parent, row, column, child, rowSpan,
                        columnSpan);
@@ -164,7 +179,7 @@ struct MiniQtInvoker : ObjectForwarder<IMiniQtHelper>
   {
     InvokeVoidFunction(&IMiniQtHelper::AddWidget, parent, child);
   }
-  void InsertWidget(QWidget *parent, int index, QWidget *child)
+  void InsertWidget(QWidget *parent, int32_t index, QWidget *child)
   {
     InvokeVoidFunction(&IMiniQtHelper::InsertWidget, parent, index, child);
   }
@@ -180,7 +195,7 @@ struct MiniQtInvoker : ObjectForwarder<IMiniQtHelper>
     return InvokeRetFunction<rdcstr>(&IMiniQtHelper::GetWidgetText, widget);
   }
 
-  void SetWidgetFont(QWidget *widget, const rdcstr &font, int fontSize, bool bold, bool italic)
+  void SetWidgetFont(QWidget *widget, const rdcstr &font, int32_t fontSize, bool bold, bool italic)
   {
     InvokeVoidFunction(&IMiniQtHelper::SetWidgetFont, widget, font, fontSize, bold, italic);
   }
@@ -193,6 +208,14 @@ struct MiniQtInvoker : ObjectForwarder<IMiniQtHelper>
   {
     return InvokeRetFunction<bool>(&IMiniQtHelper::IsWidgetEnabled, widget);
   }
+  void SetWidgetVisible(QWidget *widget, bool visible)
+  {
+    InvokeVoidFunction(&IMiniQtHelper::SetWidgetVisible, widget, visible);
+  }
+  bool IsWidgetVisible(QWidget *widget)
+  {
+    return InvokeRetFunction<bool>(&IMiniQtHelper::IsWidgetVisible, widget);
+  }
 
   // specific widgets
 
@@ -207,6 +230,10 @@ struct MiniQtInvoker : ObjectForwarder<IMiniQtHelper>
   }
 
   QWidget *CreateLabel() { return InvokeRetFunction<QWidget *>(&IMiniQtHelper::CreateLabel); }
+  void SetLabelImage(QWidget *widget, const bytebuf &data, int32_t width, int32_t height, bool alpha)
+  {
+    InvokeVoidFunction(&IMiniQtHelper::SetLabelImage, widget, data, width, height, alpha);
+  }
   QWidget *CreateOutputRenderingWidget()
   {
     return InvokeRetFunction<QWidget *>(&IMiniQtHelper::CreateOutputRenderingWidget);
@@ -243,7 +270,7 @@ struct MiniQtInvoker : ObjectForwarder<IMiniQtHelper>
     return InvokeRetFunction<bool>(&IMiniQtHelper::IsWidgetChecked, checkableWidget);
   }
 
-  QWidget *CreateSpinbox(int decimalPlaces, double step)
+  QWidget *CreateSpinbox(int32_t decimalPlaces, double step)
   {
     return InvokeRetFunction<QWidget *>(&IMiniQtHelper::CreateSpinbox, decimalPlaces, step);
   }
@@ -387,6 +414,7 @@ struct CaptureContextInvoker : ObjectForwarder<ICaptureContext>
   virtual bool IsCaptureLocal() override { return m_Obj.IsCaptureLocal(); }
   virtual bool IsCaptureTemporary() override { return m_Obj.IsCaptureTemporary(); }
   virtual bool IsCaptureLoading() override { return m_Obj.IsCaptureLoading(); }
+  virtual ReplayStatus GetFatalError() override { return m_Obj.GetFatalError(); }
   virtual rdcstr GetCaptureFilename() override { return m_Obj.GetCaptureFilename(); }
   virtual CaptureModifications GetCaptureModifications() override
   {
@@ -404,19 +432,16 @@ struct CaptureContextInvoker : ObjectForwarder<ICaptureContext>
   }
   virtual uint32_t CurSelectedEvent() override { return m_Obj.CurSelectedEvent(); }
   virtual uint32_t CurEvent() override { return m_Obj.CurEvent(); }
-  virtual const DrawcallDescription *CurSelectedDrawcall() override
+  virtual const ActionDescription *CurSelectedAction() override
   {
-    return m_Obj.CurSelectedDrawcall();
+    return m_Obj.CurSelectedAction();
   }
-  virtual const DrawcallDescription *CurDrawcall() override { return m_Obj.CurDrawcall(); }
-  virtual const DrawcallDescription *GetFirstDrawcall() override
+  virtual const ActionDescription *CurAction() override { return m_Obj.CurAction(); }
+  virtual const ActionDescription *GetFirstAction() override { return m_Obj.GetFirstAction(); }
+  virtual const ActionDescription *GetLastAction() override { return m_Obj.GetLastAction(); }
+  virtual const rdcarray<ActionDescription> &CurRootActions() override
   {
-    return m_Obj.GetFirstDrawcall();
-  }
-  virtual const DrawcallDescription *GetLastDrawcall() override { return m_Obj.GetLastDrawcall(); }
-  virtual const rdcarray<DrawcallDescription> &CurDrawcalls() override
-  {
-    return m_Obj.CurDrawcalls();
+    return m_Obj.CurRootActions();
   }
   virtual ResourceDescription *GetResource(ResourceId id) override { return m_Obj.GetResource(id); }
   virtual const rdcarray<ResourceDescription> &GetResources() override
@@ -433,14 +458,14 @@ struct CaptureContextInvoker : ObjectForwarder<ICaptureContext>
   {
     return m_Obj.HasResourceCustomName(id);
   }
-  virtual int ResourceNameCacheID() override { return m_Obj.ResourceNameCacheID(); }
+  virtual int32_t ResourceNameCacheID() override { return m_Obj.ResourceNameCacheID(); }
   virtual TextureDescription *GetTexture(ResourceId id) override { return m_Obj.GetTexture(id); }
   virtual const rdcarray<TextureDescription> &GetTextures() override { return m_Obj.GetTextures(); }
   virtual BufferDescription *GetBuffer(ResourceId id) override { return m_Obj.GetBuffer(id); }
   virtual const rdcarray<BufferDescription> &GetBuffers() override { return m_Obj.GetBuffers(); }
-  virtual const DrawcallDescription *GetDrawcall(uint32_t eventId) override
+  virtual const ActionDescription *GetAction(uint32_t eventId) override
   {
-    return m_Obj.GetDrawcall(eventId);
+    return m_Obj.GetAction(eventId);
   }
   virtual bool OpenRGPProfile(const rdcstr &filename) override
   {
@@ -450,7 +475,7 @@ struct CaptureContextInvoker : ObjectForwarder<ICaptureContext>
   virtual const SDFile &GetStructuredFile() override { return m_Obj.GetStructuredFile(); }
   virtual WindowingSystem CurWindowingSystem() override { return m_Obj.CurWindowingSystem(); }
   virtual const rdcarray<DebugMessage> &DebugMessages() override { return m_Obj.DebugMessages(); }
-  virtual int UnreadMessageCount() override { return m_Obj.UnreadMessageCount(); }
+  virtual int32_t UnreadMessageCount() override { return m_Obj.UnreadMessageCount(); }
   virtual void MarkMessagesRead() override { return m_Obj.MarkMessagesRead(); }
   virtual rdcstr GetNotes(const rdcstr &key) override { return m_Obj.GetNotes(key); }
   virtual rdcarray<EventBookmark> GetBookmarks() override { return m_Obj.GetBookmarks(); }
@@ -515,9 +540,13 @@ struct CaptureContextInvoker : ObjectForwarder<ICaptureContext>
   {
     return InvokeRetFunction<bool>(&ICaptureContext::IsResourceReplaced, id);
   }
-  virtual void RegisterReplacement(ResourceId id) override
+  virtual ResourceId GetResourceReplacement(ResourceId id) override
   {
-    InvokeVoidFunction(&ICaptureContext::RegisterReplacement, id);
+    return InvokeRetFunction<ResourceId>(&ICaptureContext::GetResourceReplacement, id);
+  }
+  virtual void RegisterReplacement(ResourceId from, ResourceId to) override
+  {
+    InvokeVoidFunction(&ICaptureContext::RegisterReplacement, from, to);
   }
   virtual void UnregisterReplacement(ResourceId id) override
   {
@@ -718,11 +747,11 @@ struct CaptureContextInvoker : ObjectForwarder<ICaptureContext>
                                     const rdcstrpairs &files, ShaderEncoding shaderEncoding,
                                     ShaderCompileFlags flags,
                                     IShaderViewer::SaveCallback saveCallback,
-                                    IShaderViewer::CloseCallback closeCallback) override
+                                    IShaderViewer::RevertCallback revertCallback) override
   {
     return InvokeRetFunction<IShaderViewer *>(&ICaptureContext::EditShader, id, stage, entryPoint,
                                               files, shaderEncoding, flags, saveCallback,
-                                              closeCallback);
+                                              revertCallback);
   }
 
   virtual IShaderViewer *DebugShader(const ShaderBindpointMapping *bind,
@@ -738,6 +767,11 @@ struct CaptureContextInvoker : ObjectForwarder<ICaptureContext>
     return InvokeRetFunction<IShaderViewer *>(&ICaptureContext::ViewShader, shader, pipeline);
   }
 
+  virtual IShaderMessageViewer *ViewShaderMessages(ShaderStageMask stages) override
+  {
+    return InvokeRetFunction<IShaderMessageViewer *>(&ICaptureContext::ViewShaderMessages, stages);
+  }
+
   virtual IBufferViewer *ViewBuffer(uint64_t byteOffset, uint64_t byteSize, ResourceId id,
                                     const rdcstr &format = "") override
   {
@@ -758,7 +792,7 @@ struct CaptureContextInvoker : ObjectForwarder<ICaptureContext>
                                                          stage, slot, idx);
   }
 
-  virtual IPixelHistoryView *ViewPixelHistory(ResourceId texID, int x, int y,
+  virtual IPixelHistoryView *ViewPixelHistory(ResourceId texID, uint32_t x, uint32_t y,
                                               const TextureDisplay &display) override
   {
     return InvokeRetFunction<IPixelHistoryView *>(&ICaptureContext::ViewPixelHistory, texID, x, y,
@@ -797,17 +831,16 @@ PythonShell::PythonShell(ICaptureContext &ctx, QWidget *parent)
   QObject::connect(ui->lineInput, &RDLineEdit::keyPress, this, &PythonShell::interactive_keypress);
   QObject::connect(ui->helpSearch, &RDLineEdit::keyPress, this, &PythonShell::helpSearch_keypress);
 
-  ui->lineInput->setFont(QFontDatabase::systemFont(QFontDatabase::FixedFont));
-  ui->interactiveOutput->setFont(QFontDatabase::systemFont(QFontDatabase::FixedFont));
-  ui->scriptOutput->setFont(QFontDatabase::systemFont(QFontDatabase::FixedFont));
-  ui->helpText->setFont(QFontDatabase::systemFont(QFontDatabase::FixedFont));
+  ui->lineInput->setFont(Formatter::FixedFont());
+  ui->interactiveOutput->setFont(Formatter::FixedFont());
+  ui->scriptOutput->setFont(Formatter::FixedFont());
+  ui->helpText->setFont(Formatter::FixedFont());
 
   ui->lineInput->setAcceptTabCharacters(true);
 
   scriptEditor = new ScintillaEdit(this);
 
-  scriptEditor->styleSetFont(
-      STYLE_DEFAULT, QFontDatabase::systemFont(QFontDatabase::FixedFont).family().toUtf8().data());
+  scriptEditor->styleSetFont(STYLE_DEFAULT, Formatter::FixedFont().family().toUtf8().data());
 
   scriptEditor->setMarginLeft(4.0 * devicePixelRatioF());
   scriptEditor->setMarginWidthN(0, 32.0 * devicePixelRatioF());
@@ -902,6 +935,62 @@ PythonContext *PythonShell::GetScriptContext()
   return scriptContext;
 }
 
+void PythonShell::SetScriptText(rdcstr script)
+{
+  scriptEditor->setText(script.c_str());
+}
+
+bool PythonShell::LoadScriptFromFilename(rdcstr filename)
+{
+  if(!filename.isEmpty())
+  {
+    QFile f(filename);
+    if(f.open(QIODevice::ReadOnly | QIODevice::Text))
+    {
+      scriptEditor->setText(f.readAll().data());
+      return true;
+    }
+  }
+
+  return false;
+}
+
+rdcstr PythonShell::GetScriptText()
+{
+  return scriptEditor->getText(scriptEditor->textLength() + 1).data();
+}
+
+void PythonShell::RunScript()
+{
+  PythonContext *context = newContext();
+
+  ANALYTIC_SET(UIFeatures.PythonInterop, true);
+
+  ui->outputHelpTabs->setCurrentIndex(0);
+
+  ui->scriptOutput->clear();
+
+  QString script = QString::fromUtf8(scriptEditor->getText(scriptEditor->textLength() + 1));
+
+  enableButtons(false);
+
+  LambdaThread *thread = new LambdaThread([this, script, context]() {
+
+    scriptContext = context;
+    context->executeString(lit("script.py"), script);
+    scriptContext = NULL;
+
+    GUIInvoke::call(this, [this, context]() {
+      context->Finish();
+      enableButtons(true);
+    });
+  });
+
+  thread->setName(lit("Python script"));
+  thread->selfDelete(true);
+  thread->start();
+}
+
 void PythonShell::on_execute_clicked()
 {
   QString command = ui->lineInput->text();
@@ -966,18 +1055,9 @@ void PythonShell::on_openScript_clicked()
   QString filename = RDDialog::getOpenFileName(this, tr("Open Python Script"), QString(),
                                                tr("Python scripts (*.py)"));
 
-  if(!filename.isEmpty())
+  if(!LoadScriptFromFilename(filename))
   {
-    QFile f(filename);
-    if(f.open(QIODevice::ReadOnly | QIODevice::Text))
-    {
-      scriptEditor->setText(f.readAll().data());
-    }
-    else
-    {
-      RDDialog::critical(this, tr("Error loading script"),
-                         tr("Couldn't open path %1.").arg(filename));
-    }
+    RDDialog::critical(this, tr("Error loading script"), tr("Couldn't open path %1.").arg(filename));
   }
 }
 
@@ -1015,33 +1095,7 @@ void PythonShell::on_saveScript_clicked()
 
 void PythonShell::on_runScript_clicked()
 {
-  PythonContext *context = newContext();
-
-  ANALYTIC_SET(UIFeatures.PythonInterop, true);
-
-  ui->outputHelpTabs->setCurrentIndex(0);
-
-  ui->scriptOutput->clear();
-
-  QString script = QString::fromUtf8(scriptEditor->getText(scriptEditor->textLength() + 1));
-
-  enableButtons(false);
-
-  LambdaThread *thread = new LambdaThread([this, script, context]() {
-
-    scriptContext = context;
-    context->executeString(lit("script.py"), script);
-    scriptContext = NULL;
-
-    GUIInvoke::call(this, [this, context]() {
-      context->Finish();
-      enableButtons(true);
-    });
-  });
-
-  thread->setName(lit("Python script"));
-  thread->selfDelete(true);
-  thread->start();
+  RunScript();
 }
 
 void PythonShell::on_abortRun_clicked()
