@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -74,15 +74,25 @@ class ShaderViewer : public QFrame, public IShaderViewer, public ICaptureViewer
   Q_OBJECT
 
 public:
-  static IShaderViewer *EditShader(ICaptureContext &ctx, ResourceId id, ShaderStage stage,
-                                   const QString &entryPoint, const rdcstrpairs &files,
-                                   ShaderEncoding shaderEncoding, ShaderCompileFlags flags,
-                                   IShaderViewer::SaveCallback saveCallback,
-                                   IShaderViewer::CloseCallback closeCallback, QWidget *parent)
+  typedef std::function<void(ShaderViewer *viewer, bool closed)> ModifyCallback;
+
+  static ShaderViewer *LoadEditor(ICaptureContext &ctx, QVariantMap data,
+                                  IShaderViewer::SaveCallback saveCallback,
+                                  IShaderViewer::RevertCallback revertCallback,
+                                  ModifyCallback modifyCallback, QWidget *parent);
+  QVariantMap SaveEditor();
+
+  static ShaderViewer *EditShader(ICaptureContext &ctx, ResourceId id, ShaderStage stage,
+                                  const QString &entryPoint, const rdcstrpairs &files,
+                                  ShaderEncoding shaderEncoding, ShaderCompileFlags flags,
+                                  IShaderViewer::SaveCallback saveCallback,
+                                  IShaderViewer::RevertCallback revertCallback,
+                                  ModifyCallback modifyCallback, QWidget *parent)
   {
     ShaderViewer *ret = new ShaderViewer(ctx, parent);
     ret->m_SaveCallback = saveCallback;
-    ret->m_CloseCallback = closeCallback;
+    ret->m_RevertCallback = revertCallback;
+    ret->m_ModifyCallback = modifyCallback;
     ret->editShader(id, stage, entryPoint, files, shaderEncoding, flags);
     return ret;
   }
@@ -110,12 +120,16 @@ public:
   virtual uint32_t CurrentStep() override;
   virtual void SetCurrentStep(uint32_t step) override;
 
-  virtual void ToggleBreakpoint(int instruction = -1) override;
+  virtual void ToggleBreakpointOnInstruction(int32_t instruction = -1) override;
+  virtual void ToggleBreakpointOnDisassemblyLine(int32_t disassemblyLine) override;
+  virtual void RunForward() override;
 
   virtual void ShowErrors(const rdcstr &errors) override;
 
   virtual void AddWatch(const rdcstr &variable) override;
 
+  virtual rdcstrpairs GetCurrentFileContents() override;
+
   // ICaptureViewer
   void OnCaptureLoaded() override;
   void OnCaptureClosed() override;
@@ -126,6 +140,8 @@ private slots:
   // automatic slots
   void on_findReplace_clicked();
   void on_refresh_clicked();
+  void on_unrefresh_clicked();
+  void on_resetEdits_clicked();
   void on_intView_clicked();
   void on_floatView_clicked();
   void on_debugToggle_clicked();
@@ -146,6 +162,7 @@ private slots:
   void watch_keyPress(QKeyEvent *event);
   void performFind();
   void performFindAll();
+  void resultsDoubleClick(int position, int line);
   void performReplace();
   void performReplaceAll();
 
@@ -153,6 +170,7 @@ private slots:
   void snippet_selectedMip();
   void snippet_selectedSlice();
   void snippet_selectedSample();
+  void snippet_selectedRange();
   void snippet_selectedType();
   void snippet_samplers();
   void snippet_resources();
@@ -160,23 +178,19 @@ private slots:
   void disasm_tooltipShow(int x, int y);
   void disasm_tooltipHide(int x, int y);
 
-public slots:
-  bool stepBack();
-  bool stepNext();
-  void runToCursor();
-  void runToSample();
-  void runToNanOrInf();
-  void runBack();
-  void run();
-
 private:
   explicit ShaderViewer(ICaptureContext &ctx, QWidget *parent = 0);
   void editShader(ResourceId id, ShaderStage stage, const QString &entryPoint,
                   const rdcstrpairs &files, ShaderEncoding shaderEncoding, ShaderCompileFlags flags);
   void debugShader(const ShaderBindpointMapping *bind, const ShaderReflection *shader,
                    ResourceId pipeline, ShaderDebugTrace *trace, const QString &debugContext);
+
   bool eventFilter(QObject *watched, QEvent *event) override;
 
+  QAction *MakeExecuteAction(QString name, const QIcon &icon, QString tooltip, QKeySequence shortcut);
+
+  void MarkModification();
+
   void PopulateCompileTools();
   void PopulateCompileToolParameters();
   bool ProcessIncludeDirectives(QString &source, const rdcstrpairs &files);
@@ -207,11 +221,13 @@ private:
   ShaderBindpointMapping m_Mapping;
   const ShaderReflection *m_ShaderDetails = NULL;
   bool m_CustomShader = false;
+  ResourceId m_EditingShader;
   ShaderCompileFlags m_Flags;
   QList<ShaderEncoding> m_Encodings;
   ShaderStage m_Stage;
   QString m_DebugContext;
   ResourceId m_Pipeline;
+  rdcarray<rdcstr> m_PipelineTargets;
   ScintillaEdit *m_DisassemblyView = NULL;
   QFrame *m_DisassemblyToolbar = NULL;
   QWidget *m_DisassemblyFrame = NULL;
@@ -248,13 +264,22 @@ private:
   } m_FindState;
 
   SaveCallback m_SaveCallback;
-  CloseCallback m_CloseCallback;
+  RevertCallback m_RevertCallback;
+  ModifyCallback m_ModifyCallback;
+
+  bool m_Modified = true;
+  bool m_Saved = false;
 
   ShaderDebugTrace *m_Trace = NULL;
   rdcarray<ShaderDebugState> m_States;
   size_t m_CurrentStateIdx = 0;
   rdcarray<ShaderVariable> m_Variables;
 
+  // true when debugging while we're populating the initial trace. Lets us queue up commands and
+  // process them once we've initialised properly
+  bool m_DeferredInit = false;
+  rdcarray<std::function<void(ShaderViewer *)>> m_DeferredCommands;
+
   QSemaphore m_BackgroundRunning;
 
   rdcarray<AccessedResourceData> m_AccessedResources;
@@ -264,6 +289,8 @@ private:
   rdcarray<BoundResourceArray> m_ReadWriteResources;
   QList<int> m_Breakpoints;
 
+  QList<QPair<ScintillaEdit *, int>> m_FindAllResults;
+
   static const int CURRENT_MARKER = 0;
   static const int BREAKPOINT_MARKER = 2;
   static const int FINISHED_MARKER = 4;
@@ -273,6 +300,7 @@ private:
 
   static const int INDICATOR_FINDRESULT = 0;
   static const int INDICATOR_REGHIGHLIGHT = 1;
+  static const int INDICATOR_FINDALLHIGHLIGHT = 2;
 
   QString targetName(const ShaderProcessingTool &disasm);
 
@@ -314,6 +342,18 @@ private:
 
   void find(bool down);
 
+  void updateEditState();
+
+  enum StepMode
+  {
+    StepInto,
+    StepOver,
+    StepOut,
+  };
+
+  bool step(bool forward, StepMode mode);
+
+  void runToCursor(bool forward);
   void runTo(QVector<size_t> runToInstructions, bool forward,
              ShaderEvents condition = ShaderEvents::NoEvent);
 
