@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -60,7 +60,7 @@ struct Following
 
   bool operator==(const Following &o);
   bool operator!=(const Following &o);
-  static void GetDrawContext(ICaptureContext &ctx, bool &copy, bool &clear, bool &compute);
+  static void GetActionContext(ICaptureContext &ctx, bool &copy, bool &clear, bool &compute);
 
   int GetHighestMip(ICaptureContext &ctx);
   int GetFirstArraySlice(ICaptureContext &ctx);
@@ -143,7 +143,7 @@ public:
 
   Subresource GetSelectedSubresource() override;
   void SetSelectedSubresource(Subresource sub) override;
-  void GotoLocation(int x, int y) override;
+  void GotoLocation(uint32_t x, uint32_t y) override;
   DebugOverlay GetTextureOverlay() override;
   void SetTextureOverlay(DebugOverlay overlay) override;
 
@@ -219,8 +219,6 @@ private slots:
   void thumb_clicked(QMouseEvent *);
   void thumb_doubleClicked(QMouseEvent *);
   void texContextItem_triggered();
-  void showUnused_triggered();
-  void showEmpty_triggered();
 
   void zoomOption_returnPressed();
 
@@ -248,7 +246,7 @@ private:
 
   void UI_UpdateStatusText();
   void UI_UpdateTextureDetails();
-  void UI_OnTextureSelectionChanged(bool newdraw);
+  void UI_OnTextureSelectionChanged(bool newAction);
 
   void UI_SetHistogramRange(const TextureDescription *tex, CompType typeCast);
 
@@ -307,6 +305,8 @@ private:
 
   void ShowGotoPopup();
 
+  bool ShouldFlipForGL();
+
   void UI_UpdateFittedScale();
   void UI_SetScale(float s);
   void UI_SetScale(float s, int x, int y);
@@ -330,15 +330,13 @@ private:
   int m_PrevFirstArraySlice = -1;
   int m_PrevHighestMip = -1;
 
-  bool m_ShowEmpty = false;
-  bool m_ShowUnused = false;
-
   bool m_Visualise = false;
   bool m_NoRangePaint = false;
   bool m_RangePoint_Dirty = false;
 
   ResourceId m_LockedId;
   QMap<ResourceId, QWidget *> m_LockedTabs;
+  int m_ResourceCacheID = -1;
 
   TextureGoto *m_Goto;
 
