@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -25,6 +25,7 @@
 #pragma once
 
 #include <QAbstractScrollArea>
+#include <QStylePainter>
 #include "Code/Interface/QRDInterface.h"
 
 class TimelineBar : public QAbstractScrollArea, public ITimelineBar, public ICaptureViewer
@@ -37,7 +38,7 @@ public:
 
   QSize minimumSizeHint() const override;
 
-  // IStatisticsViewer
+  // ITimelineBar
   QWidget *Widget() override { return this; }
   void HighlightResourceUsage(ResourceId id) override;
   void HighlightHistory(ResourceId id, const rdcarray<PixelModification> &history) override;
@@ -68,12 +69,12 @@ private:
     bool expanded = false;
 
     QVector<Marker> children;
-    QVector<uint32_t> draws;
+    QVector<uint32_t> actions;
   };
 
   QVector<Marker> m_RootMarkers;
-  QVector<uint32_t> m_RootDraws;
-  QVector<uint32_t> m_Draws;
+  QVector<uint32_t> m_RootActions;
+  QVector<uint32_t> m_Actions;
 
   ResourceId m_ID;
   QString m_HistoryTarget;
@@ -82,11 +83,11 @@ private:
   QString m_UsageTarget;
   QList<EventUsage> m_UsageEvents;
 
-  const qreal margin = 2.0;
+  const qreal margin = 3.0;
   const qreal borderWidth = 1.0;
   const QString eidAxisTitle = lit("EID:");
-  const int dataBarHeight = 18;
-  const int highlightingExtra = 12;
+  const int dataBarHeight = 16;
+  const int highlightingMargin = 2;
 
   int m_eidAxisLabelStep = 0;
   qreal m_eidAxisLabelTextWidth = 0;
@@ -108,9 +109,10 @@ private:
 
   uint32_t eventAt(qreal x);
   qreal offsetOf(uint32_t eid);
-  uint32_t processDraws(QVector<Marker> &markers, QVector<uint32_t> &draws,
-                        const rdcarray<DrawcallDescription> &curDraws);
-  void paintMarkers(QPainter &p, const QVector<Marker> &markers, const QVector<uint32_t> &draws,
+  uint32_t processActions(QVector<Marker> &markers, QVector<uint32_t> &actions,
+                          const rdcarray<ActionDescription> &curActions);
+  void drawLine(QStylePainter &p, QPointF start, QPointF end);
+  void paintMarkers(QPainter &p, const QVector<Marker> &markers, const QVector<uint32_t> &actions,
                     QRectF markerRect);
   Marker *findMarker(QVector<Marker> &markers, QRectF markerRect, QPointF pos);
 };
