@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -96,7 +96,7 @@ rdcstr GetDefaultActivityForPackage(const rdcstr &deviceID, const rdcstr &packag
   {
     line.trim();
 
-    if(line.beginsWith("name="))
+    if(line.beginsWith("name=") && !line.contains("com.android"))
     {
       return line.substr(5);
     }
@@ -381,7 +381,7 @@ ReplayStatus InstallRenderDocServer(const rdcstr &deviceID)
 
     rdcstr apkpath = paths[i] + apk + ".apk";
 
-    if(FileIO::exists(apkpath.c_str()))
+    if(FileIO::exists(apkpath))
     {
       apksFolder = paths[i];
       RDCLOG("APKs found: %s", apksFolder.c_str());
@@ -408,12 +408,25 @@ ReplayStatus InstallRenderDocServer(const rdcstr &deviceID)
 
     apk += GetRenderDocPackageForABI(abi) + ".apk";
 
-    if(!FileIO::exists(apk.c_str()))
+    if(!FileIO::exists(apk))
       RDCWARN(
           "%s missing - ensure you build all ABIs your device can support for full compatibility",
           apk.c_str());
 
-    Process::ProcessResult adbInstall = adbExecCommand(deviceID, "install -r -g \"" + apk + "\"");
+    rdcstr api =
+        Android::adbExecCommand(deviceID, "shell getprop ro.build.version.sdk").strStdout.trimmed();
+
+    int apiVersion = atoi(api.c_str());
+
+    Process::ProcessResult adbInstall;
+    if(apiVersion >= 30)
+    {
+      adbInstall = adbExecCommand(deviceID, "install -r -g --force-queryable \"" + apk + "\"");
+    }
+    else
+    {
+      adbInstall = adbExecCommand(deviceID, "install -r -g \"" + apk + "\"");
+    }
 
     RDCLOG("Installed package '%s', checking for success...", apk.c_str());
 
@@ -553,7 +566,7 @@ struct AndroidRemoteServer : public RemoteServer
   }
 
   virtual rdcpair<ReplayStatus, IReplayController *> OpenCapture(
-      uint32_t proxyid, const char *filename, const ReplayOptions &opts,
+      uint32_t proxyid, const rdcstr &filename, const ReplayOptions &opts,
       RENDERDOC_ProgressCallback progress) override
   {
     ResetAndroidSettings();
@@ -575,9 +588,9 @@ struct AndroidRemoteServer : public RemoteServer
   }
 
   virtual rdcstr GetHomeFolder() override { return ""; }
-  virtual rdcarray<PathEntry> ListFolder(const char *path) override
+  virtual rdcarray<PathEntry> ListFolder(const rdcstr &path) override
   {
-    if(path[0] == 0 || (path[0] == '/' && path[1] == 0))
+    if(path.empty() || (path[0] == '/' && path.size() == 1))
     {
       SCOPED_TIMER("Fetching android packages and activities");
 
@@ -761,7 +774,8 @@ struct AndroidRemoteServer : public RemoteServer
     }
   }
 
-  virtual ExecuteResult ExecuteAndInject(const char *a, const char *w, const char *c,
+  virtual ExecuteResult ExecuteAndInject(const rdcstr &packageAndActivity, const rdcstr &,
+                                         const rdcstr &intentArgs,
                                          const rdcarray<EnvironmentModification> &env,
                                          const CaptureOptions &opts) override;
 
@@ -1029,11 +1043,21 @@ struct AndroidController : public IDeviceProtocolHandler
       Android::adbForwardPorts(dev.portbase, deviceID, 0, 0, false);
       Android::ResetCaptureSettings(deviceID);
 
+      // make Oculus' on device vulkan validation layer available for load
+      Android::adbExecCommand(
+          deviceID,
+          "shell setprop debug.oculus.usepackagedvvl." RENDERDOC_ANDROID_PACKAGE_BASE ".arm32 1");
+      Android::adbExecCommand(
+          deviceID,
+          "shell setprop debug.oculus.usepackagedvvl." RENDERDOC_ANDROID_PACKAGE_BASE ".arm64 1");
+
       rdcstr package = GetRenderDocPackageForABI(abis.back());
 
+      rdcstr folderName = Android::GetFolderName(deviceID);
+
       // push settings file into our folder
       Android::adbExecCommand(deviceID, "push \"" + FileIO::GetAppFolderFilename("renderdoc.conf") +
-                                            "\" /sdcard/Android/data/" + package +
+                                            "\" /sdcard/Android/" + folderName + package +
                                             "/files/renderdoc.conf");
 
       // launch the last ABI, as the 64-bit version where possible, or 32-bit version where not.
@@ -1114,15 +1138,13 @@ void AndroidRemoteServer::ShutdownConnection()
   RemoteServer::ShutdownConnection();
 }
 
-ExecuteResult AndroidRemoteServer::ExecuteAndInject(const char *a, const char *w, const char *c,
+ExecuteResult AndroidRemoteServer::ExecuteAndInject(const rdcstr &packageAndActivity,
+                                                    const rdcstr &, const rdcstr &intentArgs,
                                                     const rdcarray<EnvironmentModification> &env,
                                                     const CaptureOptions &opts)
 {
   LazilyStartLogcatThread();
 
-  rdcstr packageAndActivity = a && a[0] ? a : "";
-  rdcstr intentArgs = c && c[0] ? c : "";
-
   // we spin up a thread to Ping() every second, since starting a package can block for a long time.
   int32_t done = 0;
   Threading::ThreadHandle pingThread = Threading::CreateThread([&done, this]() {
@@ -1379,7 +1401,7 @@ ExecuteResult AndroidRemoteServer::ExecuteAndInject(const char *a, const char *w
     {
       // Check if the target app has started yet and we can connect to it.
       ITargetControl *control = RENDERDOC_CreateTargetControl(
-          (AndroidController::m_Inst.GetProtocolName() + "://" + m_deviceID).c_str(), ret.ident,
+          AndroidController::m_Inst.GetProtocolName() + "://" + m_deviceID, ret.ident,
           "testConnection", false);
       if(control)
       {
