@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -107,15 +107,16 @@ typedef enum RENDERDOC_CaptureOption {
   // 0 - no callstacks are captured
   eRENDERDOC_Option_CaptureCallstacks = 3,
 
-  // When capturing CPU callstacks, only capture them from drawcalls.
+  // When capturing CPU callstacks, only capture them from actions.
   // This option does nothing without the above option being enabled
   //
   // Default - disabled
   //
-  // 1 - Only captures callstacks for drawcall type API events.
+  // 1 - Only captures callstacks for actions.
   //     Ignored if CaptureCallstacks is disabled
   // 0 - Callstacks, if enabled, are captured for every event.
   eRENDERDOC_Option_CaptureCallstacksOnlyDraws = 4,
+  eRENDERDOC_Option_CaptureCallstacksOnlyActions = 4,
 
   // Specify a delay in seconds to wait for a debugger to attach, after
   // creating or injecting into a process, before continuing to allow it to run.
@@ -524,6 +525,15 @@ typedef uint32_t(RENDERDOC_CC *pRENDERDOC_EndFrameCapture)(RENDERDOC_DevicePoint
 typedef uint32_t(RENDERDOC_CC *pRENDERDOC_DiscardFrameCapture)(RENDERDOC_DevicePointer device,
                                                                RENDERDOC_WindowHandle wndHandle);
 
+// Requests that the replay UI show itself (if hidden or not the current top window). This can be
+// used in conjunction with IsTargetControlConnected and LaunchReplayUI to intelligently handle
+// showing the UI after making a capture.
+//
+// This will return 1 if the request was successfully passed on, though it's not guaranteed that
+// the UI will be on top in all cases depending on OS rules. It will return 0 if there is no current
+// target control connection to make such a request, or if there was another error
+typedef uint32_t(RENDERDOC_CC *pRENDERDOC_ShowReplayUI)();
+
 //////////////////////////////////////////////////////////////////////////////////////////////////
 // RenderDoc API versions
 //
@@ -548,6 +558,8 @@ typedef enum RENDERDOC_Version {
   eRENDERDOC_API_Version_1_3_0 = 10300,    // RENDERDOC_API_1_3_0 = 1 03 00
   eRENDERDOC_API_Version_1_4_0 = 10400,    // RENDERDOC_API_1_4_0 = 1 04 00
   eRENDERDOC_API_Version_1_4_1 = 10401,    // RENDERDOC_API_1_4_1 = 1 04 01
+  eRENDERDOC_API_Version_1_4_2 = 10402,    // RENDERDOC_API_1_4_2 = 1 04 02
+  eRENDERDOC_API_Version_1_5_0 = 10500,    // RENDERDOC_API_1_5_0 = 1 05 00
 } RENDERDOC_Version;
 
 // API version changelog:
@@ -574,8 +586,10 @@ typedef enum RENDERDOC_Version {
 // 1.4.0 - Added feature: DiscardFrameCapture() to discard a frame capture in progress and stop
 //         capturing without saving anything to disk.
 // 1.4.1 - Refactor: Renamed Shutdown to RemoveHooks to better clarify what is happening
+// 1.4.2 - Refactor: Renamed 'draws' to 'actions' in callstack capture option.
+// 1.5.0 - Added feature: ShowReplayUI() to request that the replay UI show itself if connected
 
-typedef struct RENDERDOC_API_1_4_1
+typedef struct RENDERDOC_API_1_5_0
 {
   pRENDERDOC_GetAPIVersion GetAPIVersion;
 
@@ -647,17 +661,22 @@ typedef struct RENDERDOC_API_1_4_1
 
   // new function in 1.4.0
   pRENDERDOC_DiscardFrameCapture DiscardFrameCapture;
-} RENDERDOC_API_1_4_1;
 
-typedef RENDERDOC_API_1_4_1 RENDERDOC_API_1_0_0;
-typedef RENDERDOC_API_1_4_1 RENDERDOC_API_1_0_1;
-typedef RENDERDOC_API_1_4_1 RENDERDOC_API_1_0_2;
-typedef RENDERDOC_API_1_4_1 RENDERDOC_API_1_1_0;
-typedef RENDERDOC_API_1_4_1 RENDERDOC_API_1_1_1;
-typedef RENDERDOC_API_1_4_1 RENDERDOC_API_1_1_2;
-typedef RENDERDOC_API_1_4_1 RENDERDOC_API_1_2_0;
-typedef RENDERDOC_API_1_4_1 RENDERDOC_API_1_3_0;
-typedef RENDERDOC_API_1_4_1 RENDERDOC_API_1_4_0;
+  // new function in 1.5.0
+  pRENDERDOC_ShowReplayUI ShowReplayUI;
+} RENDERDOC_API_1_5_0;
+
+typedef RENDERDOC_API_1_5_0 RENDERDOC_API_1_0_0;
+typedef RENDERDOC_API_1_5_0 RENDERDOC_API_1_0_1;
+typedef RENDERDOC_API_1_5_0 RENDERDOC_API_1_0_2;
+typedef RENDERDOC_API_1_5_0 RENDERDOC_API_1_1_0;
+typedef RENDERDOC_API_1_5_0 RENDERDOC_API_1_1_1;
+typedef RENDERDOC_API_1_5_0 RENDERDOC_API_1_1_2;
+typedef RENDERDOC_API_1_5_0 RENDERDOC_API_1_2_0;
+typedef RENDERDOC_API_1_5_0 RENDERDOC_API_1_3_0;
+typedef RENDERDOC_API_1_5_0 RENDERDOC_API_1_4_0;
+typedef RENDERDOC_API_1_5_0 RENDERDOC_API_1_4_1;
+typedef RENDERDOC_API_1_5_0 RENDERDOC_API_1_4_2;
 
 //////////////////////////////////////////////////////////////////////////////////////////////////
 // RenderDoc API entry point
