@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2021 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -47,7 +47,7 @@ struct CaptureOptions
   DOCUMENT(R"(Encode the current options to a string suitable for passing around between processes.
 
 :return: The encoded string, suitable for passing to :meth:`DecodeFromString`.
-:rtype: ``str``
+:rtype: str
 )");
   inline rdcstr EncodeAsString() const
   {
@@ -63,16 +63,21 @@ struct CaptureOptions
     return optstr;
   }
 
-  DOCUMENT("Decode the options from a string, as returned by :meth:`EncodeAsString`.");
-  inline void DecodeFromString(const rdcstr &str)
+  DOCUMENT(R"(Decode the options from a string, as returned by :meth:`EncodeAsString`. Updates this
+object in place.
+
+:param str encoded: The encoded string, as returned by :meth:`EncodeAsString`.
+)");
+  inline void DecodeFromString(const rdcstr &encoded)
   {
-    if(str.size() < sizeof(CaptureOptions))
+    if(encoded.size() < sizeof(CaptureOptions))
       return;
 
     // serialise from string with two chars per byte
     byte *b = (byte *)this;
     for(size_t i = 0; i < sizeof(CaptureOptions); i++)
-      *(b++) = byte(((byte(str[i * 2 + 0] - 'a') & 0xf) << 4) | (byte(str[i * 2 + 1] - 'a') & 0xf));
+      *(b++) = byte(((byte(encoded[i * 2 + 0] - 'a') & 0xf) << 4) |
+                    (byte(encoded[i * 2 + 1] - 'a') & 0xf));
   }
 
   DOCUMENT(R"(Allow the application to enable vsync.
@@ -116,16 +121,16 @@ Default - disabled
 )");
   bool captureCallstacks;
 
-  DOCUMENT(R"(When capturing CPU callstacks, only capture them from drawcalls.
+  DOCUMENT(R"(When capturing CPU callstacks, only capture them from actions.
 This option does nothing if :data:`captureCallstacks` is not enabled.
 
 Default - disabled
 
-``True`` - Only captures callstacks for drawcall type API events.
+``True`` - Only captures callstacks for actions.
 
 ``False`` - Callstacks, if enabled, are captured for every event.
 )");
-  bool captureCallstacksOnlyDraws;
+  bool captureCallstacksOnlyActions;
 
   DOCUMENT(R"(Specify a delay in seconds to wait for a debugger to attach, after
 creating or injecting into a process, before continuing to allow it to run.
